/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * main.cc
 * Copyright (C) 2021 MicroEM JSC <support@microem.ru>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name ``MicroEM JSC'' nor the name of any other
 *    contributor may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 
 * CLESCAR IS PROVIDED BY MicroEM JSC ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MicroEM JSC OR ANY OTHER CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "config.h"

#ifdef ENABLE_NLS
#  include <libintl.h>
#endif

#include "interface.h"
#include "reader.h"
#include "search.h"
#include "mifare_classic.h"

/* For testing propose use the local (not installed) ui file */
/* #define UI_FILE PACKAGE_DATA_DIR"/ui/clescar.ui" */
#define UI_FILE "src/clescar.ui"

void* m_pReader;

extern char card_info[256];
extern bool m_isOpened;

Glib::Dispatcher m_DispatcherNewCard;
Glib::Dispatcher m_DispatcherNoCards;

Gtk::Window* main_win = 0;
Gtk::Notebook* notebook;
extern Gtk::Button* btn_interface_open;
extern Gtk::ListBox* list_cards;
extern Gtk::CheckButton* chk_enable_beep;
extern Gtk::Button* btn_get_version;
extern Gtk::Button* btn_get_serial;
extern Gtk::Entry* edt_version;
extern Gtk::Entry* edt_serial;
extern Gtk::Button* btn_beep;
extern Gtk::SpinButton* spin_beeps;
extern Gtk::Button* btn_led;
extern Gtk::ComboBoxText* cmb_color;
extern Gtk::ComboBoxText* cmb_post_color;
extern Gtk::Entry* edt_mifare_key;
extern Gtk::RadioButton* rad_mifare_key_a;
extern Gtk::RadioButton* rad_mifare_key_b;
extern Gtk::Button* btn_mifare_load_key;
extern Gtk::SpinButton* spin_mifare_sector;
extern Gtk::Button* btn_mifare_auth;
extern Gtk::SpinButton* spin_mifare_block;
extern Gtk::Button* btn_mifare_read;
extern Gtk::Entry* edt_mifare_data;
extern Gtk::Button* btn_mifare_write;

extern void on_btnOpenInterface_clicked();
extern void on_btn_get_version_clicked();
extern void on_btn_get_serial_clicked();
extern void on_btn_beep_clicked();
extern void on_btn_led_clicked();
extern void on_btn_mifare_load_key_clicked();
extern void on_btn_mifare_auth_clicked();
extern void on_btn_mifare_read_clicked();
extern void on_btn_mifare_write_clicked();

void on_notebook_SwitchPage(Gtk::Widget* page, unsigned int page_num);

void on_newCard();

void on_noCards();

int test(int argc, char *argv[]);

int main(int argc, char *argv[])
{
    LONG Status;
	
	Status = CLSCRF_Create( &m_pReader );

    if( Status == 0 )
        printf("Interface is created successfully\n");
    else
        printf("Interface is not created\n");

	//test(argc, argv);
	
	Gtk::Main kit(argc, argv);

	//Load the Glade file and instiate its widgets:
	Glib::RefPtr<Gtk::Builder> builder;
	try
	{
		builder = Gtk::Builder::create_from_file(UI_FILE);
	}
	catch (const Glib::FileError & ex)
	{
		std::cerr << ex.what() << std::endl;
		return 1;
	}
	builder->get_widget("main_window", main_win);
    builder->get_widget("btn_interface_open", btn_interface_open);
	builder->get_widget("notebook", notebook);
	builder->get_widget("list_cards", list_cards);
	builder->get_widget("chk_enable_beep", chk_enable_beep);
    builder->get_widget("btn_get_version", btn_get_version);
    builder->get_widget("btn_get_serial", btn_get_serial);
    builder->get_widget("edt_version", edt_version);
    builder->get_widget("edt_serial", edt_serial);
    builder->get_widget("btn_beep", btn_beep);
	builder->get_widget("spin_beeps", spin_beeps);
	builder->get_widget("btn_led", btn_led);
	builder->get_widget("cmb_color", cmb_color);
	builder->get_widget("cmb_post_color", cmb_post_color);
    builder->get_widget("edt_mifare_key", edt_mifare_key);
	builder->get_widget("rad_mifare_key_a", rad_mifare_key_a);
	builder->get_widget("rad_mifare_key_b", rad_mifare_key_b);
	builder->get_widget("btn_mifare_load_key", btn_mifare_load_key);
	builder->get_widget("spin_mifare_sector", spin_mifare_sector);
	builder->get_widget("btn_mifare_auth", btn_mifare_auth);
	builder->get_widget("spin_mifare_block", spin_mifare_block);
	builder->get_widget("btn_mifare_read", btn_mifare_read);
    builder->get_widget("edt_mifare_data", edt_mifare_data);
	builder->get_widget("btn_mifare_write", btn_mifare_write);
	
	spin_beeps->set_range(1, 100);
	spin_beeps->set_value(1);
	spin_beeps->set_increments(1, 10);

	spin_mifare_sector->set_range(0, 47);
	spin_mifare_sector->set_value(1);
	spin_mifare_sector->set_increments(1, 10);

	spin_mifare_block->set_range(0, 15);
	spin_mifare_block->set_value(0);
	spin_mifare_block->set_increments(1, 10);

	m_DispatcherNewCard.connect(sigc::ptr_fun(&on_newCard));
	m_DispatcherNoCards.connect(sigc::ptr_fun(&on_noCards));
	
	notebook->signal_switch_page().connect( sigc::ptr_fun(&on_notebook_SwitchPage) );

	btn_interface_open->signal_clicked().connect( sigc::ptr_fun(&on_btnOpenInterface_clicked) );
    btn_get_serial->signal_clicked().connect( sigc::ptr_fun(&on_btn_get_serial_clicked) );
    btn_get_version->signal_clicked().connect( sigc::ptr_fun(&on_btn_get_version_clicked) );
    btn_beep->signal_clicked().connect( sigc::ptr_fun(&on_btn_beep_clicked) );
    btn_led->signal_clicked().connect( sigc::ptr_fun(&on_btn_led_clicked) );
    btn_mifare_load_key->signal_clicked().connect( sigc::ptr_fun(&on_btn_mifare_load_key_clicked) );
	btn_mifare_auth->signal_clicked().connect( sigc::ptr_fun(&on_btn_mifare_auth_clicked) );
	btn_mifare_read->signal_clicked().connect( sigc::ptr_fun(&on_btn_mifare_read_clicked) );
	btn_mifare_write->signal_clicked().connect( sigc::ptr_fun(&on_btn_mifare_write_clicked) );
		
	std::thread thread(search_thread_entry, m_pReader, list_cards);
	thread.detach();
	//std::thread(search_thread_entry).detach();
	
	if (main_win)
	{
		kit.run(*main_win);
	}

	stop_search = true;

	if (m_isOpened)
	{
		Status = CLSCRF_Close( m_pReader );

		if( Status == 0 )
		    printf("Interface is closed successfully\n");
		else
		    printf("Interface is not closed\n");
	}
		
    /*Status = CLSCRF_Destroy( &m_pReader );

    if( Status == 0 )
        printf("Interface is destroyed successfully\n");
    else
        printf("Interface is not destroyed\n");*/
	
	return 0;
}

void on_noCards()
{
	for (int i = 0; i < list_cards->get_children().size(); i++)
	{
		Gtk::Widget* element = list_cards->get_children()[i];
		list_cards->remove(*element);
		delete element;
	}
}

void on_newCard()
{
	Gtk::Label* label = new Gtk::Label();
	label->set_text(card_info);
	list_cards->add(*label);
	//list_cards->insert(*label, -1);
	list_cards->show_all();
}

void
on_notebook_SwitchPage(Gtk::Widget* page, unsigned int page_num)
{
	std::cout << "Switching page to " << page_num << std::endl;

	if (page_num == 2)
	{
		pause_search = false;
	}
	else
	{
		pause_search = true;
	}
}

int
test (int argc, char *argv[])
{
    LONG Status;
    unsigned char Buf[300];
    DWORD i, n;
    char Err;
    bool  InterfaceRS = true;
    DWORD PortIndex = 0;
//========================================================================
//  ...>Demo.exe       - äëÿ ðàáîòû ñî ñ÷èòûâàòåëåì UEM RS íà COM1
//  ...>Demo.exe RS 1  - äëÿ ðàáîòû ñî ñ÷èòûâàòåëåì UEM RS íà COM2
//  ...>Demo.exe RS 2  - äëÿ ðàáîòû ñî ñ÷èòûâàòåëåì UEM RS íà COM3 è ò.ä.
//  ...>Demo.exe USB   - äëÿ ðàáîòû ñ åäèíñòâåííûì ñ÷èòûâàòåëåì UEM USB
//  ...>Demo.exe USB 1 - äëÿ ðàáîòû ñî âòîðûì ñ÷èòûâàòåëåì UEM USB
//  ...>Demo.exe USB 2 - äëÿ ðàáîòû ñ òðåòüèì ñ÷èòûâàòåëåì UEM USB è ò.ä.
//========================================================================
    if( argc > 1 && strcmp( argv[1], "USB" ) == 0 )
            InterfaceRS = false;

    if( argc > 2 )
        PortIndex = atoi( argv[2] );

    printf("\nCONTACTLESS CARD READER's FUNCTION LIBRARY's Demo Program\n");
    printf("\n");

    Status = CLSCRF_Create( &m_pReader );

    if( Status == 0 )
        printf("Interface is created successfully\n");
    else
        printf("Interface is not created\n");

    if( InterfaceRS )
        Status = CLSCRF_OpenRS( m_pReader, PortIndex, 9600, 0 );
    else
        Status = CLSCRF_OpenUSB( m_pReader, PortIndex, 0 );

    if( Status == 0 )
    {
        Status = CLSCRF_Sound( m_pReader, 2 );
        if( InterfaceRS )
            printf("Interface COM%d is opened successfully\n", PortIndex+1);
        else
            printf("Interface USB(%d) is opened successfully\n", PortIndex);
    }
    else if( InterfaceRS )
        printf("Interface COM%d is not opened\n", PortIndex+1);
    else
        printf("Interface USB(%d) is not opened\n", PortIndex);

//    if( InterfaceRS )
    {
        Status = CLSCRF_Mfrc_On( m_pReader );

        if( Status == 0 )
            printf("Reader-IC is turn on successfully\n");
        else
            printf("Reader-IC is not turn on\n");
    }

    printf("Mfrc_Version        ");
    Status = CLSCRF_Get_Mfrc_Version( m_pReader, Buf );
    if( Status == 0 )
        for( i = 0; i < 6; i++ )
            printf("%02X", Buf[i] );
    else
        printf("Sorry, error 0x%08X", (UINT)Status );
    printf("\n");

    printf("Mfrc_Serial_Number  ");
    Status = CLSCRF_Get_Mfrc_Serial_Number( m_pReader, Buf );
    if( Status == 0 )
        for( i = 0; i < 4; i++ )
            printf("%02X", Buf[3-i] );
    else
        printf("Sorry, error 0x%08X", (UINT)Status );
    printf("\n");

    Sleep( 1000 );

    CLSCRF_Mfrc_Rf_Off_On( m_pReader, 1 );

    printf("Activate_Idle_A\n");
    Status = CLSCRF_Activate_Idle_A( m_pReader,
                                     &Buf[0],// pbATQ 2 bytes
                                     &Buf[2],// pbSAK 1 byte
                                     &Buf[3],// pbUID max 10 bytes
                                     &n );
    if( Status == 0 )
    {
        CLSCRF_Sound( m_pReader, 1 );
        printf("ATQ   %02X%02X\n", Buf[1], Buf[0] );
        printf("SAK   %02X\n", Buf[2] );
        printf("UID   " );
        for( i = 0; i < n; i++ )
            printf("%02X", Buf[i+3] );
        printf("\n");
        CLSCRF_Halt_A( m_pReader );
    }
    else
    {
        printf("Sorry, error 0x%08X\n", (UINT)Status );
        if( Status == 0x80100001 )
        {
            CLSCRF_GetLastInternalError( m_pReader, (BYTE*)&Err );
            printf("Internal Error = %d\n", Err );
        }
    }

//    if( InterfaceRS )
    {
    Status = CLSCRF_Mfrc_Off( m_pReader );

    if( Status == 0 )
        printf("Reader-IC is turn off successfully\n");
    else
        printf("Reader-IC is not turn off\n");
    }

    Status = CLSCRF_Close( m_pReader );

    if( Status == 0 )
        printf("Interface is closed successfully\n");
    else
        printf("Interface is not closed\n");

    Status = CLSCRF_Destroy( &m_pReader );

    if( Status == 0 )
        printf("Interface is destroyed successfully\n");
    else
        printf("Interface is not destroyed\n");

    printf("\n");
    //printf( "Press Enter key ..." );
    //getchar();

	return 0;
}
