package com.example.uemotglibrary;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Bundle;

import com.example.uemreaderlibrary.UemReader;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.android.material.tabs.TabLayout;

import androidx.viewpager.widget.ViewPager;
import androidx.appcompat.app.AppCompatActivity;

import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.example.uemotglibrary.ui.main.SectionsPagerAdapter;

public class MainActivity extends AppCompatActivity {

    private Activity mMainActivity;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mMainActivity = this;
        setContentView(R.layout.activity_main);
        SectionsPagerAdapter sectionsPagerAdapter = new SectionsPagerAdapter(this, getSupportFragmentManager());
        ViewPager viewPager = findViewById(R.id.view_pager);
        viewPager.setAdapter(sectionsPagerAdapter);
        TabLayout tabs = findViewById(R.id.tabs);
        tabs.setupWithViewPager(viewPager);
        FloatingActionButton fab = findViewById(R.id.fab);

        final UemReader uemR = new UemReader(mMainActivity);

        uemR.Bluetooth.RegisterCallbacks(new UemReader.Callbacks() {
            @Override
            public void onBluetoothReaderFound(String readerName) {
                uemR.Bluetooth.StopDiscovery();
                uemR.Connect(readerName);
            }

            @Override
            public void onBluetoothReaderConnected() {
                uemR.Reader.Sound(3);
            }

            @Override
            public void onBluetoothResponse(byte[] response) {
                if (response != null) {
                    final String bufReceived = UemReader.Helper.byteArrayToHexString(response);
                    mMainActivity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(getApplicationContext(),
                                    bufReceived, Toast.LENGTH_LONG)
                                    .show();
                        }
                    });
                }
                uemR.Disconnect();
            }
        });

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //new Thread(new Runnable() {
                //    public void run() {
                        /*mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
                        HashMap<String, UsbDevice> deviceList = mUsbManager.getDeviceList();
                        UsbDevice device = deviceList.get("deviceName");

                        TextView devs = findViewById(R.id.title);
                        devs.setText(String.valueOf(deviceList.size()));
                        for (UsbDevice usbDevice: mUsbManager.getDeviceList().values()) {
                            devs.setText(usbDevice.getDeviceName());
                            mDevice = usbDevice;
                        }
                        byte[] bytes = UemReader.Helper.wrapCommand(new byte[]{0x05, 0x02});//Beep Twice

                        int TIMEOUT = 0;
                        boolean forceClaim = true;

                        if (_device == null)
                            return;

                        UsbInterface interfaceUEM = _device.getInterface(0);
                        UsbEndpoint endpointBulkIn = interfaceUEM.getEndpoint(0);
                        UsbEndpoint endpointBulkOut = interfaceUEM.getEndpoint(1);
                        UsbDeviceConnection connection = usbManager.openDevice(_device);
                        connection.claimInterface(interfaceUEM, forceClaim);
                        connection.bulkTransfer(endpointBulkOut, bytes, bytes.length, TIMEOUT); //do in another thread
                        byte[] capture = { 0x14, 0x00, 0x00, 0x00, 0x10, 0x00, 0x0E, 0x10,
                                0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                                0x00, 0x00 };
                        final int nReceived = connection.bulkTransfer(endpointBulkIn, capture, capture.length, TIMEOUT);
                        //final String bufReceived = UemReader.byteArrayToHexString(Arrays.copyOf(capture, nReceived));
                        final String bufReceived = UemReader.Helper.byteArrayToHexString(UemReader.Helper.unwrapResponse(Arrays.copyOf(capture, nReceived)));
                        mainActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Toast.makeText(getApplicationContext(),
                                        bufReceived, Toast.LENGTH_LONG)
                                        .show();
                            }});
                        //final byte[] command = UemReader.Helper.wrapCommand(new byte[]{0x05, 0x02});*/

                        String[] arReaders = uemR.USB.Discover();
                        if (arReaders.length > 0) {
                            if (uemR.Connect(arReaders[0])) {
                                /*byte[] recBytes = uemR.DirectIO_Reader(new byte[]{0x05, 0x02});
                                if (recBytes != null) {
                                    final String bufReceived = UemReader.Helper.byteArrayToHexString(recBytes);
                                    mMainActivity.runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            Toast.makeText(getApplicationContext(),
                                                    bufReceived, Toast.LENGTH_LONG)
                                                    .show();
                                        }
                                    });
                                }*/
                                uemR.Reader.Sound(3);
                                uemR.Disconnect();
                            }
                        }

                        uemR.Bluetooth.StartDiscovery();

                //    }
                //}).start();
            }
        });

        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        filter.addAction(ACTION_USB_PERMISSION);

        registerReceiver(usbReceiver, filter);
    }

    private static final String ACTION_USB_PERMISSION =
            "com.android.example.USB_PERMISSION";
    private final BroadcastReceiver usbReceiver = new BroadcastReceiver() {

        public void onReceive(Context context, Intent intent) {
            //TextView devs = findViewById(R.id.devs);
            //devs.setText("Received broadcast!");
            String action = intent.getAction();
            Log.d("---", "Received Broadcast: "+action);
            if (ACTION_USB_PERMISSION.equals(action)) {
                synchronized (this) {
                    UsbDevice device = (UsbDevice) intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);

                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
                        if (device != null) {
                            //call method to set up device communication
                            Toast.makeText(getApplicationContext(),
                                    "ACTION_USB_PERMISSION" , Toast.LENGTH_LONG)
                                    .show();
                            //mDevice = device;
                            //devs.setText(device.getDeviceName());
                        }
                    } else {
                        Log.d("---", "permission denied for device " + device);
                    }
                }
            }
            if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action) || UsbManager.ACTION_USB_ACCESSORY_ATTACHED.equals(action)) {
                UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                if (device != null) {
                    Toast.makeText(getApplicationContext(),
                            "ACTION_USB_DEVICE_ATTACHED" , Toast.LENGTH_LONG)
                            .show();
                }
                Log.d("---", "USB Connected..");
            }
            if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action) || UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) {
                UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                if (device != null) {
                    // call your method that cleans up and closes communication with the device
                    Toast.makeText(getApplicationContext(),
                            "ACTION_USB_DEVICE_DETACHED" , Toast.LENGTH_LONG)
                            .show();
                }
                Log.d("---", "USB Disconnected..");
            }
        }
    };


}