// Transfer.h: interface for the CTransfer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TRANSFER_H__14085D87_B9A2_4C05_B2F3_B4456CC197A9__INCLUDED_)
#define AFX_TRANSFER_H__14085D87_B9A2_4C05_B2F3_B4456CC197A9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#include "WinSCard.h"
#include "Serial.h"
//#include "IOUART.h"
#include "IOUSB.h"
#include "../Clscrfl.h"
#endif // _MSC_VER > 1000

#define TX_BUF_SIZE     300
#define RX_BUF_SIZE     300

#define DEFAULT_IO_TIMEOUT       1000
#define MINIMUM_IO_TIMEOUT         50
#define MAXIMUM_IO_TIMEOUT   86400000

#define F_OSC   24000000

#define B_SPECIAL               0xFD // 
#define B_START                 0xFD // 
#define B_STOP                  0xFE // 
#define B_STUFF                 0xFF // 

#define ICODE_PACKET_SIZE     32
#define ICODE_BLOCK_SIZE       4
#define ICODE_EAS_DATA_SIZE   32

typedef union _STATE_TRANS
{
    unsigned char n;
    struct flags{
        unsigned RX_start : 1;  //   
        unsigned RX_stuff : 1;  //  -
        unsigned RX_addr  : 1;  //   
        unsigned _X_busy  : 1;  // //   
        unsigned _X_OK    : 1;  // // 
        unsigned _X_Err   : 1;  // // 
        unsigned _TX_last : 1;
        unsigned _reset   : 1;
    } b;
} STATE_TRANS;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class CTransceiveBuffer 
{
public:
	CTransceiveBuffer ();
	virtual ~CTransceiveBuffer ();

    DWORD GetLength() { return m_Count; }
    void Clear() { m_Count = 0; }
    bool AddByte( unsigned char Value );
    bool AddBulk( const unsigned char *Data, DWORD Size );
    bool GetBulk( int nIndex, unsigned char *Data, DWORD *pSize );
    BYTE operator []( int nIndex ) const;

protected:
    DWORD           m_Count;
    unsigned char   m_Buf[TX_BUF_SIZE];
};
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

typedef void  (*LPFN_TRANS_ERROR)( int Port, UINT Oper, DWORD Err, void *Context  );
typedef void  (*LPFN_TRANS_RX)( int Port, BYTE *Data, int Size, void *Context );

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class CTransfer
{
public:
	CTransfer();
	virtual ~CTransfer();

    CSerialPort     m_ComPort;
//    CIOUART     m_ComPort;
    CIOUSB      m_USB;

//    void Transmit( unsigned char *Data, int Size );
    bool Open( DWORD LogLevel, 
               int Port = 0, 
               int Baud = 9600 );
    bool IsOpened() { return m_ComPort.IsOpened() || m_USB.IsOpened(); }
    DWORD GetTimeout() { return m_Timeout; }
    void SetTimeout( DWORD NewTimeout );

    bool Close( void );
//#if ADDRESS_RS_485
    BYTE GetIOAddress() { return m_IO_Addr; }
    void SetIOAddress( BYTE Addr ) { m_IO_Addr = Addr; }
//#endif
    HANDLE  m_EvIOComplete;
    DWORD m_Error;
    LONG    m_LongError;
    void OnError( int Port, UINT Oper, DWORD Err );
    bool OnRX( int Port, BYTE *RXBuff, int Size );

    int m_PortNumber;
    int m_Baudrate;
    unsigned char m_SendBuffer[ RX_BUF_SIZE ];

    CTransceiveBuffer m_TrBuf;
    CTransceiveBuffer m_RcBuf;

    LONG SingleTransceive();

protected:

    unsigned char m_RX_buf[ RX_BUF_SIZE ];
    unsigned char m_TX_buf[ TX_BUF_SIZE ];
    DWORD m_RX_cnt;
    DWORD m_TX_cnt;
    DWORD m_Timeout;

//#if ADDRESS_RS_485
    unsigned char m_IO_Addr;
//#endif
    unsigned char m_Cmd_ID;
    STATE_TRANS   m_state;

    unsigned short m_RX_FCS;
    unsigned short m_TX_FCS;

    void RX_Reset( void );
    void CalculateFCS( unsigned short &FCS, unsigned char b );
    void PostByte( unsigned char b );
    void ByteStuff( unsigned char b );
    char OnReceive( unsigned char inp );
};
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

static void CommError( int Port, UINT Oper, DWORD Err, LPVOID Context ) \
    { ((CTransfer*)Context)->OnError( Port, Oper, Err ); }

static void CommRX( int Port, BYTE *RXBuff, int Size, LPVOID Context ) \
    { ((CTransfer*)Context)->OnRX( Port, RXBuff, Size ); }


#endif // !defined(AFX_TRANSFER_H__14085D87_B9A2_4C05_B2F3_B4456CC197A9__INCLUDED_)
