# pip install pyusb
import usb
import time

USB_VID = 0xc251
USB_PID = 0x130a
USB_EP_WRITE = 0x02
USB_EP_READ = 0x82
	
reattach = False
command_id = 0
command_code = 0



# dev.set_configuration() 
# cfg = dev.get_active_configuration() 

# interface_number = cfg[(0,0)].bInterfaceNumber 
# alternate_settting = usb.control.get_interface(dev, interface_number) 
# intf = usb.util.find_descriptor(cfg, bInterfaceNumber = interface_number, 
#                             bAlternateSetting = alternate_settting) 

# epw = usb.util.find_descriptor(intf,custom_match = \
#       lambda e: \
#     usb.util.endpoint_direction(e.bEndpointAddress) == \
#     usb.util.ENDPOINT_OUT) 
# epr = usb.util.find_descriptor(intf,custom_match = \
#       lambda e: \
#     usb.util.endpoint_direction(e.bEndpointAddress) == \
#     usb.util.ENDPOINT_IN) 
# epw.write('\xFD\x00\x00\x05\x01\xEF\x93\xFE')
# 239 147
# 05 00 102 130
# ret = epr.read(8)
# print(ret)

# CRC-16/X-25  \x00\x00\x05\x01

def openReader():
	global reattach
	dev = usb.core.find(idVendor=USB_VID, idProduct=USB_PID)
	
	if dev is None:
		return none
	
	if dev.is_kernel_driver_active(0):
		reattach = True
		dev.detach_kernel_driver(0)
	
	return dev
    
    
def closeReader(dev):
	global reattach
	usb.util.dispose_resources(dev)

	if reattach:
		dev.attach_kernel_driver(0) 
    
def crc16(data: bytes, poly=0x8408):
    '''
    CRC-16-CCITT Algorithm
    '''
    data = bytearray(data)
    crc = 0xFFFF
    for b in data:
        cur_byte = 0xFF & b
        for _ in range(0, 8):
            if (crc & 0x0001) ^ (cur_byte & 0x0001):
                crc = (crc >> 1) ^ poly
            else:
                crc >>= 1
            cur_byte >>= 1
    crc = (~crc & 0xFFFF)
    crc = (crc << 8) | ((crc >> 8) & 0xFF)
    
    return crc & 0xFFFF
    
def byteStuff(data: bytes):
	out_data = bytes(0)
	for b in data:
		if b < 0xFD:
			out_data += bytes([b])
		else:
			out_data += bytes([0xFF])
			out_data += bytes([0xFF - b])
	return out_data

def unByteStuff(data: bytes):
	out_data = bytes(0)
	subtract_next = False
	for b in data:
		if subtract_next:
			out_data += bytes([0xFF - b])
			subtract_next = False
			continue
		if b == 0xFF:
			subtract_next = True
			continue
		out_data += bytes([b])
	return out_data
				
def wrapCommand(command: bytes):
	global command_id
	global command_code
	command_code = command[0]
	frame_payload = bytes([0]) + bytes([command_id]) + command
	command_id += 1
	frame_payload_crc = crc16(frame_payload)
	frame_payload += bytes([(frame_payload_crc >> 8) & 0xFF, frame_payload_crc & 0xFF])
	frame_payload_bytestuffed = byteStuff(frame_payload)
	frame = bytes([0xFD]) + frame_payload_bytestuffed + bytes([0xFE])
	return frame

def unWrapResponse(frame):
	global command_id
	if len(frame) < 8:
		print("Communication error: more frame bytes needed!")
		return None
	if frame[0] != 0xFD:
		print("Communication error: frame start missing!")
		return None
	if frame[-1] != 0xFE:
		print("Communication error: frame end missing!")
		return None
	frame_payload = unByteStuff(frame[1:-1])
	if len(frame_payload) < 6:
		print("Communication error: more payload bytes needed!")
		return None
	frame_payload_crc = crc16(frame_payload[:-2])
	if (frame_payload_crc >> 8) & 0xFF != frame_payload[-2] or \
		frame_payload_crc & 0xFF != frame_payload[-1]:
		print("Communication error: incorrect crc16!")
		return None
	if frame_payload[1] != command_id - 1:
		print("Communication error: incorrect command index!")
		return None
	if frame_payload[2] != command_code:
		print("Communication error: incorrect command code!")
		return None
	return frame_payload[3:-2]
		
def transcieve(dev, command: bytes):
	# dev.write(USB_EP_WRITE,b'\xFD\x00\x00\x05\x01\xEF\x93\xFE')
	frame_command = wrapCommand(command)
	dev.write(USB_EP_WRITE, frame_command)
	frame_response = dev.read(USB_EP_READ,100000,1000)
	response = unWrapResponse(frame_response)
	return response

	
reader = openReader()
if reader is None:
	exit

beep_command = b'\x05\x01'
	
response = transcieve(reader, beep_command)
print(response)

time.sleep(1)

response = transcieve(reader, beep_command)
print(response)

closeReader(reader)    


