﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MifarePlusCS
{
    public partial class Form1 : Form
    {
        IntPtr _readerHandle = new IntPtr();
        byte _lastErr = 0x00;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Clscrfl_SAM.Create(ref _readerHandle);
        }

        private void btnOpenUSB_Click(object sender, EventArgs e)
        {
            Clscrfl_SAM.OpenUSB(_readerHandle);
            Clscrfl_SAM.Blink(_readerHandle);
            Clscrfl_SAM.Sound(_readerHandle);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (_lastErr == 0x00)
            {
                Clscrfl_SAM.ReaderOn(_readerHandle);
                toolStripStatusLabel1.Text = "Считыватель готов к работе";
            }
            else
            {
                toolStripStatusLabel1.Text = "Ошибка коммуникации со считывателем!";
            }
        }

        private void btnSearchCard_Click(object sender, EventArgs e)
        {
            Clscrfl_SAM.CLSCRF_Mfrc_Rf_Off_On(_readerHandle, 100);
            byte[] UID = Clscrfl_SAM.ConnectToCard(_readerHandle);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (_lastErr == 0x00)
            {
                String strUID = Clscrfl_SAM.ByteArrayToHexString(UID);
                toolStripStatusLabel1.Text = "Найдена карта " + strUID;
                lblCardUID.Text = strUID; 
            }
            else if (_lastErr == 0xFF)
            {
                toolStripStatusLabel1.Text = "Нет карты в поле!";
                return;
            }
            else
            {
                toolStripStatusLabel1.Text = "Ошибка!";
                return;
            }
            uint res = Clscrfl_SAM.RequestAnswerToSelect(_readerHandle);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (res == 0 && _lastErr != 0x00)
            {
                toolStripStatusLabel1.Text = "Это не карта Mifare Plus!";
            }
        }

        private void btnAuthenticate_Click(object sender, EventArgs e)
        {
            byte[] key = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            Clscrfl_SAM.WriteFlashValue(_readerHandle, 0, key);
            uint res = Clscrfl_SAM.MifarePlus_AuthenticateA(_readerHandle, byte.Parse(edtSector.Text), 0);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (res == 0 && _lastErr == 0x00)
            {
                toolStripStatusLabel1.Text = "Аутентификация прошла успешно";
            }
            else
            {
                toolStripStatusLabel1.Text = "Ошибка аутентификации!";
            }
        }

        private void btnReadData_Click(object sender, EventArgs e)
        {
            byte[] data = Clscrfl_SAM.MifarePlusReadData(_readerHandle, 0x00, 0x00, byte.Parse(edtSector.Text), byte.Parse(edtBlock.Text), 1);

            uint res = Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (res == 0 && _lastErr == 0x00)
            {
                toolStripStatusLabel1.Text = "Чтение прошло успешно";

                edtBlockData.Text = Clscrfl_SAM.ByteArrayToHexString(data);
            }
            else
            {
                toolStripStatusLabel1.Text = "Ошибка чтения!";
            }
        }

        private void btnWriteData_Click(object sender, EventArgs e)
        {
            byte[] data = Clscrfl_SAM.HexStringToByteArray(edtBlockData.Text);

            uint res = Clscrfl_SAM.MifarePlusWriteData(_readerHandle, 0x00, 0x00, byte.Parse(edtSector.Text), byte.Parse(edtBlock.Text), 1, data);

            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (res == 0 && _lastErr == 0x00)
            {
                toolStripStatusLabel1.Text = "Запись прошла успешно";
            }
            else
            {
                toolStripStatusLabel1.Text = "Ошибка записи!";
            }
        }

        private void btnPersonalizeCard_Click(object sender, EventArgs e)
        {
            byte[] data = new byte[16];
            Array.Clear(data, 0x00, data.Length);
            uint res = 0;

            for (byte ucSect = 0x00; ucSect < 0x28; ucSect++)
            {
                res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x4A, ucSect, 0x00, data);
                if (res != 0) break;
                res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x4B, ucSect, 0x00, data);
                if (res != 0) goto err;
            }
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0xB1, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x90, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x91, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x92, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x93, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0x94, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0xA0, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0xA1, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0xA2, 0x00, 0x00, data);
            if (res != 0) goto err;
            res = Clscrfl_SAM.MifarePlusWritePerso(_readerHandle, 0xA3, 0x00, 0x00, data);
            if (res != 0) goto err;

            res = Clscrfl_SAM.MifarePlusCommitPerso(_readerHandle);
            if (res != 0) goto err;

            Clscrfl_SAM.CLSCRF_Mfrc_Rf_Off_On(_readerHandle, 100);
            byte[] UID = Clscrfl_SAM.ConnectToCard(_readerHandle);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (_lastErr != 0) goto err;

            res = Clscrfl_SAM.RequestAnswerToSelect(_readerHandle);
            Clscrfl_SAM.CLSCRF_GetLastInternalError(_readerHandle, out _lastErr);
            if (_lastErr != 0) goto err;

            byte[] key = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
            Clscrfl_SAM.WriteFlashValue(_readerHandle, 0, key);
            res = Clscrfl_SAM.MifarePlus_Authenticate(_readerHandle, (byte)0x93, 0, 0);
            if (res != 0) goto err;

            toolStripStatusLabel1.Text = "Карта успешно переведена в режим SL3!";
            return;
err:
            toolStripStatusLabel1.Text = "Ошибка персонализации карты!";
        }
    }
}
