package ru.microem.clescar;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.scene.control.*;
import ru.microem.clscrfl.Clscrfl;

public class Controller {
    public static Controller controllerObject;
    public byte[] lastUID = null;
    public Button btnOpen;
    public ComboBox cbInterface = new ComboBox();
    public Button btnVersion;
    public Button btnSerial;
    public Button btnBeep;
    public Label lblVersion;
    public Label lblSerial;
    public Spinner spMifareClassicSector;
    public Button btnMifareClassicAuthenticateA;
    public Button btnMifareClassicAuthenticateB;
    public TextField txtMifareClassicKeyA;
    public TextField txtMifareClassicKeyB;
    public Spinner spMifareClassicRecord;
    public Button btnMifareClassicLoadKeys;
    public Spinner spMifareClassicBlock;
    public Button btnMifareClassicReadBlock;
    public Button btnMifareClassicWriteBlock;
    public TextField txtMifareClassicBlockData;
    public Label lblResult;
    public Tab tabAutosearch;
    public ListView lvCards;

    private boolean isOpened = false;
    private Clscrfl clscrfl = null;
    private TaskAutosearch taskAutosearch;
    private Thread threadAutosearch;

    public Controller() {
        //ObservableList<String> usbList = FXCollections.observableArrayList("USB0", "USB1", "USB2");
        //cbInterface.setItems(usbList);
        //cbInterface = new ComboBox(usbList);
        cbInterface.getItems().addAll(
                "USB0",
                "USB1",
                "USB2"
        );

        controllerObject = this;
        try {
            clscrfl = new Clscrfl();
            clscrfl.setVerboseOutput(true);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        taskAutosearch = new TaskAutosearch(clscrfl);
        threadAutosearch = new Thread(taskAutosearch);
        threadAutosearch.start();
    }

    public void onOpenClicked(ActionEvent actionEvent) {
        if (!isOpened) {
            try {
                int usbIndex = cbInterface.getSelectionModel().getSelectedIndex();
                clscrfl.OpenUSB(usbIndex);
                lblResult.setText("OK");
                isOpened = true;
                btnOpen.setText("Close");
                clscrfl.beep(1);
            } catch (Clscrfl.ClscrflException e) {
                lblResult.setText(e.getFullMessage());
            }
        } else {
            try {
                clscrfl.Close();
            } catch (Clscrfl.ClscrflException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            isOpened = false;
            taskAutosearch.pause();
            btnOpen.setText("Open");
        }
    }

    public void onVersionClicked(ActionEvent actionEvent) {
        try {
            byte[] version = clscrfl.getVersion();
            lblVersion.setText(Clscrfl.bytesToHex(version));
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onSerialClicked(ActionEvent actionEvent) {
        try {
            byte[] serial = clscrfl.getSerial();
            lblSerial.setText(Clscrfl.bytesToHex(serial));
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onBeepClicked(ActionEvent actionEvent) {
        try {
            clscrfl.beep(1);
        } catch (Clscrfl.ClscrflException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void onMifareClassicAuthenticateAClicked(ActionEvent actionEvent) {
        if (lastUID == null) {
            lblResult.setText("Please read a card first");
            return;
        }
        try {
            clscrfl.mifareClassicAuthenticateA(
                    (Integer)spMifareClassicSector.getValue(),
                    (Integer)spMifareClassicRecord.getValue(),
                    lastUID);
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onMifareClassicAuthenticateBClicked(ActionEvent actionEvent) {
        if (lastUID == null) {
            lblResult.setText("Please read a card first");
            return;
        }
        try {
            clscrfl.mifareClassicAuthenticateB(
                    (Integer)spMifareClassicSector.getValue(),
                    (Integer)spMifareClassicRecord.getValue(),
                    lastUID);
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onMifareClassicLoadKeysClicked(ActionEvent actionEvent) {
        try {
            byte[] keyA = Clscrfl.hexToBytes(txtMifareClassicKeyA.getText());
            clscrfl.mifareClassicLoadKeyA((Integer)spMifareClassicRecord.getValue(), keyA);
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
        try {
            byte[] keyB = Clscrfl.hexToBytes(txtMifareClassicKeyB.getText());
            clscrfl.mifareClassicLoadKeyB((Integer)spMifareClassicRecord.getValue(), keyB);
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onMifareClassicReadBlockClicked(ActionEvent actionEvent) {
        try {
            byte[] data = clscrfl.mifareClassicRead(
                    (Integer)spMifareClassicSector.getValue(),
                    (Integer)spMifareClassicBlock.getValue());
            lblResult.setText("OK");
            txtMifareClassicBlockData.setText(Clscrfl.bytesToHex(data));
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onMifareClassicWriteBlockClicked(ActionEvent actionEvent) {
        try {
            byte[] data = Clscrfl.hexToBytes(txtMifareClassicBlockData.getText());
            clscrfl.mifareClassicWrite(
                    (Integer)spMifareClassicSector.getValue(),
                    (Integer)spMifareClassicBlock.getValue(),
                    data);
            lblResult.setText("OK");
        } catch (Clscrfl.ClscrflException e) {
            lblResult.setText(e.getFullMessage());
        }
    }

    public void onAutosearchChange(Event event) {
        if (tabAutosearch.isSelected()) {
            //System.out.println("as on");
            taskAutosearch.resume();
        } else {
            //System.out.println("as off");
            taskAutosearch.pause();
        }
    }
///////////////
    protected void onWindowClosing() {
        taskAutosearch.quit();
    }

    public void delayedAddCardToList(String cardDescription) {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                Controller.controllerObject.lvCards.getItems().add(cardDescription);
            }
        });
    }

    public void delayedClearCardList() {
        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                Controller.controllerObject.lvCards.getItems().clear();
            }
        });
    }
}
