#include "StdAfx.h"

#include "Clscrfl.h"

#include <fstream>

extern std::map<PVOID, READER_SETTINGS> m_ReaderSettings;

void getKeyFromPassword(LPSTR lsPassword, DWORD dwPassLength, LPBYTE Key)
{
	BYTE pass[16], u[16], buf[16];

	if (dwPassLength <= 0)
		return;

	for (DWORD i = 0; i <= 15; i++)
		pass[i] = 0;
	for (DWORD i = 0; i <= dwPassLength - 1; i++)
		pass[i % 16] ^= lsPassword[i];

	for (DWORD i = 0; i <= 15; i++)
	{
		u[i] = 1;
		Key[i] = 0;
	}
	for (DWORD i = 0; i <= 99; i++)
	{
		aes_128_encrypt16(pass, u, buf);
		for (DWORD j = 0; j <= 15; j++)
			u[j] = buf[j];
		for (DWORD j = 0; j <= 15; j++)
			Key[j] ^= u[j];
	}
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_SaveAESKeysToFile(IN LPSTR  lsFileName,
IN LPBYTE pbIV,
IN LPBYTE pbKeys,
IN DWORD  dwKeysCount,
IN LPSTR  lsPassword)
{
	size_t fnl = strlen(lsFileName), pl = strlen(lsPassword);

	if (dwKeysCount <= 0 || fnl <= 0 || pl <= 0)
		return SCARD_E_INVALID_PARAMETER;

	std::ofstream file(lsFileName, std::ios_base::trunc | std::ios_base::binary | std::ios_base::out);
	if (!file.is_open())
		return SCARD_E_FILE_NOT_FOUND;//ERROR_INVALID_NAME;

	BYTE xorKeys[16];
	BYTE hash[16];

	for (DWORD j = 0; j <= 15; j++)
	{
		xorKeys[j] = 0x00;
		for (DWORD i = 0; i <= dwKeysCount - 1; i++)
			xorKeys[j] ^= pbKeys[i * 16 + j];
	}
	aes_128_encrypt16(xorKeys, xorKeys, hash);

	BYTE *buf = new BYTE[16 + dwKeysCount * 16];
	memcpy(buf, hash, 16);
	memcpy(buf + 16, pbKeys, dwKeysCount * 16);

	BYTE key[16];
	getKeyFromPassword(lsPassword, (DWORD)pl, key);

	BYTE *out = new BYTE[16 + dwKeysCount * 16 + 16];
	if (aes_128_encrypt_cbc_mod(buf, 16 + dwKeysCount * 16, key, pbIV, out) != 16 + dwKeysCount * 16 + 16)
	{
		file.close();
		delete[] buf;
		delete[] out;
		return SCARD_F_UNKNOWN_ERROR;//ERROR_INCORRECT_SIZE;
	}

	file.write((char *)pbIV, 16);
	if (file.fail())
	{
		file.close();
		delete[] buf;
		delete[] out;
		return SCARD_E_UNEXPECTED;//ERROR_EA_FILE_CORRUPT;
	}
	file.write((char *)out, 16 + dwKeysCount * 16 + 16);
	if (file.fail())
	{
		file.close();
		delete[] buf;
		delete[] out;
		return SCARD_E_UNEXPECTED;//ERROR_EA_FILE_CORRUPT;
	}

	file.close();
	delete[] buf;
	delete[] out;

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_GenerateAndSaveAESKeysToFile(IN LPSTR  lsFileName,
IN DWORD  dwKeysCount,
IN LPSTR  lsPassword)
{
	size_t fnl = strlen(lsFileName), pl = strlen(lsPassword);

	if (dwKeysCount <= 0 || fnl <= 0 || pl <= 0)
		return SCARD_E_INVALID_PARAMETER;

	UINT32 sr = (UINT32)time(NULL);
	srand(sr);

	BYTE pbIV[16], *pbKeys = new BYTE[dwKeysCount * 16];

	BYTE key[16];
	for (INT i = 0; i <= 15; i++)
		key[i] = 0;
	for (INT i = 0; i <= pl - 1; i++)
		key[i % 16] ^= lsPassword[i];

	BYTE *inData = new BYTE[16 * dwKeysCount + 16];
	for (DWORD i = 0; i <= 16 * dwKeysCount + 16 - 1; i++)
		inData[i] = lsPassword[i % pl];

	BYTE *data = new BYTE[16 * dwKeysCount + 32];
	aes_128_encrypt_cbc_mod(inData, 16 * dwKeysCount + 16, key, key, data);

	for (UINT i = 0; i <= 15; i++)
		pbIV[i] = (BYTE)rand() ^ data[i];

	for (UINT i = 0; i <= dwKeysCount * 16 - 1; i++)
		pbKeys[i] = (BYTE)rand() ^ data[i + 16];

	LONG Status = CLSCRF_Crypto_SaveAESKeysToFile(lsFileName, pbIV, pbKeys, dwKeysCount, lsPassword);
	delete[] pbKeys;
	delete[] inData;
	delete[] data;

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_LoadAESKeysFromFile(IN LPVOID pReader,
IN LPSTR  lsFileName,
IN LPSTR  lsPassword,
OUT LPDWORD ldKeysCount)
{
	*ldKeysCount = 0;

	if (!pReader)
		return SCARD_E_READER_UNAVAILABLE;

	size_t fnl = strlen(lsFileName), pl = strlen(lsPassword);

	if (fnl <= 0 || pl <= 0)
		return SCARD_E_INVALID_PARAMETER;

	std::ifstream file(lsFileName, std::ios_base::binary | std::ios_base::in);
	if (!file.is_open())
		return SCARD_E_FILE_NOT_FOUND;//ERROR_INVALID_NAME;

	file.seekg(0, std::ios_base::end);
	INT fsize = (INT)file.tellg();
	file.seekg(0, std::ios_base::beg);

	if (fsize < 64 || fsize % 16 != 0)
		return SCARD_F_UNKNOWN_ERROR;//ERROR_BAD_FILE_TYPE;

	BYTE pbIV[16], *data = new BYTE[fsize - 16];

	file.read((char *)pbIV, 16);
	file.read((char *)data, fsize - 16);

	BYTE key[16];
	getKeyFromPassword(lsPassword, (DWORD)pl, key);

	BYTE *hashPlusKeys = new BYTE[fsize - 32];
	if (aes_128_decrypt_cbc_mod(data, fsize - 16, key, pbIV, hashPlusKeys) != (UINT)(fsize - 32))
	{
		delete[] data;
		delete[] hashPlusKeys;
		return SCARD_F_UNKNOWN_ERROR;//ERROR_INCORRECT_SIZE;
	}

	DWORD dwKeysCount = (fsize - 48) / 16;
	BYTE xorKeys[16];
	BYTE hash[16];

	for (DWORD j = 0; j <= 15; j++)
	{
		xorKeys[j] = 0x00;
		for (DWORD i = 0; i <= dwKeysCount - 1; i++)
			xorKeys[j] ^= hashPlusKeys[i * 16 + j + 16];
	}
	aes_128_encrypt16(xorKeys, xorKeys, hash);

	if (memcmp(hash, hashPlusKeys, 16) != 0)
	{
		delete[] data;
		delete[] hashPlusKeys;
		return SCARD_E_NO_ACCESS;//ERROR_INVALID_PASSWORD;
	}

	DWORD AesKeysCount = dwKeysCount;
	BYTE *AesKeys = new BYTE[fsize - 48];
	memcpy(AesKeys, hashPlusKeys + 16, fsize - 48);

	CTransfer* pTr = (CTransfer*)pReader;
	pTr->SetAesKeys(AesKeysCount, AesKeys);

	/*printf("Keys:\n");
	for (unsigned int i = 0; i <= AesKeysCount - 1; i++)
	{
	printf("%d) ", i);
	for (int j = 0; j <= 15; j++)
	{
	if (j == 8)
	printf("\n   ");
	printf("0x%02x, ", AesKeys[i * 16 + j]);
	}
	printf("\n");
	}*/

	*ldKeysCount = AesKeysCount;

	delete[] data;
	delete[] hashPlusKeys;

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_GetAESKey(IN  LPVOID	pReader,
IN  DWORD	dwKeyIndex,
OUT LPBYTE	lpKey)
{
	if (!pReader)
		return SCARD_E_READER_UNAVAILABLE;

	DWORD keysCount;
	BYTE *keys;

	CTransfer* pTr = (CTransfer*)pReader;

	keysCount = pTr->GetAesKeys(&keys);

	memcpy(lpKey, keys + 16 * dwKeyIndex, 16);

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_GetAESKeysCount(IN LPVOID pReader,
OUT LPDWORD pdwKeysCount)
{
	if (!pReader)
		return SCARD_E_READER_UNAVAILABLE;

	if (!pdwKeysCount)
		return SCARD_E_INVALID_PARAMETER;

	DWORD keysCount;
	BYTE *keys;

	CTransfer* pTr = (CTransfer*)pReader;

	keysCount = pTr->GetAesKeys(&keys);

	*pdwKeysCount = keysCount;

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_WriteFlash_AESKeys(IN LPVOID pReader,
IN DWORD  dwStartKey,
IN DWORD  dwKeysCount)
{

	LONG Status = SCARD_S_SUCCESS;
	DWORD keysCount;
	BYTE *keys;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;

		keysCount = pTr->GetAesKeys(&keys);
		if (keysCount <= 0)
			return SCARD_E_INVALID_PARAMETER;//ERROR_RESOURCE_NOT_AVAILABLE;
		if (dwKeysCount <= 0 || dwStartKey + dwKeysCount > keysCount)
			return SCARD_E_INVALID_PARAMETER;

		for (DWORD dwAddr = dwStartKey; (dwAddr < dwStartKey + dwKeysCount) && (Status == SCARD_S_SUCCESS); dwAddr++)
			Status = CLSCRF_WriteFlashValue(pReader, dwAddr, keys + dwAddr * 16);
		//if (Status == SCARD_S_SUCCESS)
		//    Status = CLSCRF_WriteFlashValue(pReader, 0x00008000, keys);
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}


CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_SetEncryptionMode(IN LPVOID pReader,
IN BOOL fEncryptionMode)
{
	m_ReaderSettings[pReader].CurrentCryptoSettings()->fEncryptedMode = (fEncryptionMode == TRUE);

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_GetEncryptionMode(IN LPVOID pReader,
IN PBOOL pfEncryptionMode)
{
	if (pfEncryptionMode)
		*pfEncryptionMode = m_ReaderSettings[pReader].CurrentCryptoSettings()->fEncryptedMode ? TRUE : FALSE;

	return SCARD_S_SUCCESS;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_AuthenticateReader(IN LPVOID pReader,
IN BYTE ucAuthType,
IN DWORD dwKeyNumber)
{
	LONG Status = SCARD_S_SUCCESS;

	const int BUF_SIZE = 300;
	BYTE SendBuffer[BUF_SIZE];
	BYTE RecvBuffer[BUF_SIZE];
	DWORD SendLength;
	DWORD RecvLength = BUF_SIZE;

	BYTE RndA[16] = { 0 };
	BYTE RndB[16] = { 0 };
	BYTE SessionKey[16] = { 0 };
	//BYTE ucKeyVer = 0x00;
	int iKeyLength = 16;
	BYTE* pKey;
	DWORD keysCount;

	CTransfer* pTr = (CTransfer*)pReader;


	if (ucAuthType == 0x01)
	{
		keysCount = pTr->GetAesKeys(&pKey);
		if (keysCount <= 0)
			return SCARD_E_INVALID_PARAMETER;//ERROR_RESOURCE_NOT_AVAILABLE;
		if (dwKeyNumber > 0)
		{
			if (dwKeyNumber > keysCount - 1)
				return SCARD_E_INVALID_PARAMETER;
		}
		else
		{
		}

		ZeroMemory(m_ReaderSettings[pReader].CurrentCryptoSettings()->IV, 16);
	}

	iKeyLength = 16;

	SendBuffer[0] = 0x6F;
	SendBuffer[1] = (ucAuthType == 0x01) ? 0xAA : 0x00;
	SendBuffer[2] = dwKeyNumber & 0xFF;
	SendBuffer[3] = (dwKeyNumber >> 8) & 0xFF;
	SendLength = 4;

	pTr->m_TrBuf.Clear();
	pTr->m_TrBuf.AddBulk(SendBuffer, SendLength);

	Status = pTr->SingleTransceive();

	if ((ucAuthType != 0x01))//&&( Status == SCARD_S_SUCCESS ))
	{
		if (Status == SCARD_S_SUCCESS)
		{
			m_ReaderSettings[pReader].CurrentCryptoSettings()->fEncryptedMode = false;
		}
		return Status;
	}

	RecvLength = 16;
	if (!pTr->m_RcBuf.GetBulk(1, RecvBuffer, &RecvLength))
		Status = SCARD_E_COMM_DATA_LOST;

	if ((Status == SCARD_S_SUCCESS) && RecvLength)
	{

		BYTE pOut[33] = { 0 };

		//memset(&pKey[dwKeyNumber * 16], 0, 16);//test
		//BYTE testR[16] = {0x67, 0x8a, 0x3f, 0x27, 0x64, 0x1c, 0x7a, 0x55, 0x73, 0xab, 0xbb, 0xc0, 0xad, 0xec, 0x42, 0xbc}; //test
		aes_128_decrypt_cbc(
			RecvBuffer,
			16,
			&pKey[dwKeyNumber * 16],
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV,
			&pOut[16],
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV
			);

		memcpy(RndB, &pOut[16], 16);

		pOut[16 + 16] = pOut[16];

		memmove(
			pOut + 16,
			pOut + 17,
			16
			);

		for (int i = 0; i < 16; i++)
			RndA[i] = pOut[i] = (BYTE)rand();

		aes_128_encrypt_cbc(
			pOut,
			32,
			&pKey[dwKeyNumber * 16],
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV,
			&SendBuffer[4],
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV
			);

		SendBuffer[0] = 0x6F;
		SendBuffer[1] = 0xAF;
		SendBuffer[2] = dwKeyNumber & 0xFF;
		SendBuffer[3] = (dwKeyNumber >> 8) & 0xFF;

		SendLength = 36;

		RecvLength = BUF_SIZE;

		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddBulk(SendBuffer, SendLength);

		Status = pTr->SingleTransceive();

		RecvLength = 16;
		if (!pTr->m_RcBuf.GetBulk(1, RecvBuffer, &RecvLength))
			Status = SCARD_E_COMM_DATA_LOST;

		if (Status)
		{
			return Status;
		}

		aes_128_decrypt_cbc(
			RecvBuffer,
			16,
			&pKey[dwKeyNumber * 16],
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV,
			pOut,
			m_ReaderSettings[pReader].CurrentCryptoSettings()->IV
			);

		pOut[0 + 16] = pOut[0 + 16 - 1];
		memmove(
			pOut + 1,
			pOut + 0,
			16
			);
		pOut[0] = pOut[0 + 16];

		if (memcmp(RndA, pOut, 16) != 0)
		{
			return SCARD_E_CARD_UNSUPPORTED;
		}

	}

	if (Status == SCARD_S_SUCCESS)
	{
		memcpy(&SessionKey[0], &RndA[0], 4);
		memcpy(&SessionKey[4], &RndB[0], 4);
		memcpy(&SessionKey[8], &RndA[12], 4);
		memcpy(&SessionKey[12], &RndB[12], 4);

		ZeroMemory(m_ReaderSettings[pReader].CurrentCryptoSettings()->IV, 16);

		memcpy(&m_ReaderSettings[pReader].CurrentCryptoSettings()->SessionKey, SessionKey, 16);

		//CryptoSubkeys(pReader);//, SessionKey, SessionKeySize, m_DF_Settings[pReader].K1, m_DF_Settings[pReader].K2);

		m_ReaderSettings[pReader].CurrentCryptoSettings()->fEncryptedMode = true;
	}

	return Status;
}


/*
CLSCRF_API LONG CLSCRF_DECL
CLSCRF_Crypto_SetEncryptionMode(	IN LPVOID pReader,
IN DWORD  dwIVNumber,
IN DWORD  dwKeyNumber )
{
LONG Status = SCARD_S_SUCCESS;
DWORD keysCount;
BYTE *keys;

if( pReader )
{
CTransfer* pTr = (CTransfer*)pReader;

keysCount = pTr->GetAesKeys(&keys);
if (dwIVNumber > 0 && dwKeyNumber > 0)
{
if (keysCount <= 0)
return ERROR_RESOURCE_NOT_AVAILABLE;
if (dwIVNumber > keysCount - 1 || dwKeyNumber > keysCount - 1)
return SCARD_E_INVALID_PARAMETER;
}

pTr->m_TrBuf.Clear();
pTr->m_TrBuf.AddByte( 0x6F );
pTr->m_TrBuf.AddBulk( (PBYTE)&dwIVNumber, 2 );
pTr->m_TrBuf.AddBulk( (PBYTE)&dwKeyNumber, 2 );

Status = pTr->SingleTransceive();
if (Status == SCARD_S_SUCCESS)
{
pTr->ChooseAesKeys(dwIVNumber, dwKeyNumber);

pTr->SetEncryptionMode(true);
}
}
else
{
Status = SCARD_E_READER_UNAVAILABLE;
}

return Status;
}
//#endif
//=====================================================================
*/
