#include "StdAfx.h"

#include "Clscrfl.h"

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_WritePersoExplicit(IN LPVOID pReader,
IN BYTE  ucValueType,
IN BYTE  ucSectorNumber,
IN BYTE  ucBlockNumber,
IN LPBYTE pData)
{
	LONG Status = SCARD_S_SUCCESS;

	DWORD dwValueFlashAddress = 0xFFFF;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA8);
		pTr->m_TrBuf.AddByte(ucValueType);
		pTr->m_TrBuf.AddByte(ucSectorNumber);
		pTr->m_TrBuf.AddByte(ucBlockNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwValueFlashAddress, 2);
		pTr->m_TrBuf.AddBulk(pData, 16);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_CommitPerso(IN LPVOID pReader)
{
	LONG Status = SCARD_S_SUCCESS;

	DWORD dwPass = 0x87654321;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xAA);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwPass, 4);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_Authenticate(IN LPVOID pReader,
IN BYTE  ucAuthType,
IN BYTE  ucKeyType,
IN BYTE  ucSectorNumber,
IN DWORD  dwKeyFlashAddress,
IN BYTE  ucLenCap,
IN LPBYTE pbPcdCap)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA0);
		pTr->m_TrBuf.AddByte(ucAuthType);
		pTr->m_TrBuf.AddByte(ucKeyType);
		pTr->m_TrBuf.AddByte(ucSectorNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwKeyFlashAddress, 2);
		pTr->m_TrBuf.AddByte(ucLenCap);
		if (ucLenCap > 0)
			if (pbPcdCap)
				pTr->m_TrBuf.AddBulk(pbPcdCap, ucLenCap);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_AuthenticateDirect(IN LPVOID pReader,
IN BYTE  ucAuthType,
IN BYTE  ucKeyType,
IN BYTE  ucSectorNumber,
IN LPBYTE  pbKey,
IN BYTE  ucLenCap,
IN LPBYTE pbPcdCap)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xE9);
		pTr->m_TrBuf.AddByte(0x2D);
		pTr->m_TrBuf.AddByte(ucAuthType);
		pTr->m_TrBuf.AddByte(ucKeyType);
		pTr->m_TrBuf.AddByte(ucSectorNumber);
		pTr->m_TrBuf.AddBulk(pbKey, 16);
		pTr->m_TrBuf.AddByte(ucLenCap);
		if (ucLenCap > 0)
			if (pbPcdCap)
				pTr->m_TrBuf.AddBulk(pbPcdCap, ucLenCap);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_MultiBlockRead(IN LPVOID pReader,
IN BYTE ucSectorNumber,
IN BYTE ucBlockNumber,
IN BYTE ucBlocksCount,
OUT LPBYTE pbData)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		if (ucBlocksCount < 1)
			ucBlocksCount = 1;

		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA6);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress((DWORD)ucSectorNumber, (DWORD)ucBlockNumber));
		pTr->m_TrBuf.AddByte(ucBlocksCount);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
			DWORD Size = 1;

			BYTE bStatusCode = 0x00;

			if (pTr->m_RcBuf.GetBulk(0, &bStatusCode, &Size))
			{
				if (bStatusCode == 0x00)
				{
					Size = 2;

					DWORD dwLen = 0;

					if (pTr->m_RcBuf.GetBulk(1, (PBYTE)&dwLen, &Size))
					{
						if (dwLen > 0)
						{
							pTr->m_RcBuf.GetBulk(3, pbData, &dwLen);
						}
					}
				}
			}

		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_MultiBlockWrite(IN LPVOID pReader,
IN BYTE ucSectorNumber,
IN BYTE ucBlockNumber,
IN BYTE ucBlocksCount,
IN LPBYTE pbData)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		if (ucBlocksCount < 1)
			ucBlocksCount = 1;

		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA7);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress((DWORD)ucSectorNumber, (DWORD)ucBlockNumber));
		pTr->m_TrBuf.AddByte(ucBlocksCount);
		pTr->m_TrBuf.AddBulk(pbData, ucBlocksCount * 16);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_ReadData(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucValueType,
IN BYTE ucSectorNumber,
IN BYTE ucBlockNumber,
IN BYTE ucBlocksCount,
OUT LPBYTE pbData)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		if (ucBlocksCount < 1)
			ucBlocksCount = 1;

		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA4);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucValueType);
		pTr->m_TrBuf.AddByte(ucSectorNumber);
		pTr->m_TrBuf.AddByte(ucBlockNumber);
		pTr->m_TrBuf.AddByte(ucBlocksCount);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
			DWORD Size = 1;

			BYTE bStatusCode = 0x00;

			if (pTr->m_RcBuf.GetBulk(0, &bStatusCode, &Size))
			{
				if (bStatusCode == 0x00)
				{
					Size = 2;

					DWORD dwLen = 0;

					if (pTr->m_RcBuf.GetBulk(1, (PBYTE)&dwLen, &Size))
					{
						if (dwLen > 0)
						{
							pTr->m_RcBuf.GetBulk(3, pbData, &dwLen);
						}
					}
				}
			}

		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_WriteData(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucValueType,
IN BYTE ucSectorNumber,
IN BYTE ucBlockNumber,
IN BYTE ucBlocksCount,
IN LPBYTE pbData)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		if (ucBlocksCount < 1)
			ucBlocksCount = 1;

		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA5);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucValueType);
		pTr->m_TrBuf.AddByte(ucSectorNumber);
		pTr->m_TrBuf.AddByte(ucBlockNumber);
		pTr->m_TrBuf.AddByte(ucBlocksCount);
		pTr->m_TrBuf.AddBulk(pbData, (DWORD)ucBlocksCount * 16);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_Increment(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucSourceSectorNumber,
IN BYTE ucSourceBlockNumber,
IN DWORD dwValue)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xB0);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucSourceSectorNumber);
		pTr->m_TrBuf.AddByte(ucSourceBlockNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwValue, 4);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_Decrement(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucSourceSectorNumber,
IN BYTE ucSourceBlockNumber,
IN DWORD dwValue)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xB2);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucSourceSectorNumber);
		pTr->m_TrBuf.AddByte(ucSourceBlockNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwValue, 4);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_Transfer(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucDestinationSectorNumber,
IN BYTE ucDestinationBlockNumber)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xB4);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucDestinationSectorNumber);
		pTr->m_TrBuf.AddByte(ucDestinationBlockNumber);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_IncrementTransfer(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucSourceSectorNumber,
IN BYTE ucSourceBlockNumber,
IN BYTE ucDestinationSectorNumber,
IN BYTE ucDestinationBlockNumber,
IN DWORD dwValue)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xB6);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucSourceSectorNumber);
		pTr->m_TrBuf.AddByte(ucSourceBlockNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwValue, 4);
		pTr->m_TrBuf.AddByte(ucDestinationSectorNumber);
		pTr->m_TrBuf.AddByte(ucDestinationBlockNumber);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_DecrementTransfer(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucSourceSectorNumber,
IN BYTE ucSourceBlockNumber,
IN BYTE ucDestinationSectorNumber,
IN BYTE ucDestinationBlockNumber,
IN DWORD dwValue)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xB8);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucSourceSectorNumber);
		pTr->m_TrBuf.AddByte(ucSourceBlockNumber);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwValue, 4);
		pTr->m_TrBuf.AddByte(ucDestinationSectorNumber);
		pTr->m_TrBuf.AddByte(ucDestinationBlockNumber);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_Restore(IN LPVOID pReader,
IN BYTE ucEncryptionMode,
IN BYTE ucSourceSectorNumber,
IN BYTE ucSourceBlockNumber)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA1);
		pTr->m_TrBuf.AddByte(0xC2);
		pTr->m_TrBuf.AddByte(ucEncryptionMode);
		pTr->m_TrBuf.AddByte(ucSourceSectorNumber);
		pTr->m_TrBuf.AddByte(ucSourceBlockNumber);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_VirtualCardSupport(IN LPVOID pReader,
IN LPBYTE  pIID)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA2);
		pTr->m_TrBuf.AddByte(0x00);
		pTr->m_TrBuf.AddBulk(pIID, 16);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_VirtualCardSupportLast(IN LPVOID pReader,
IN LPBYTE  pIID,
IN DWORD dwKencFlashAddress,
IN DWORD dwKmacFlashAddress,
IN BYTE  ucLenCap,
IN LPBYTE pbPcdCap,
OUT PBYTE pucInfo,
OUT PBYTE pPiccCap,
OUT PBYTE pPaddedUID)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA2);
		pTr->m_TrBuf.AddByte(0x01);
		pTr->m_TrBuf.AddBulk(pIID, 16);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwKencFlashAddress, 2);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwKmacFlashAddress, 2);
		pTr->m_TrBuf.AddByte(ucLenCap);
		if (ucLenCap > 0)
			if (pbPcdCap)
				pTr->m_TrBuf.AddBulk(pbPcdCap, ucLenCap);

		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
			DWORD dwSize = 1;

			BYTE bStatusCode = 0x00;

			if (pTr->m_RcBuf.GetBulk(0, &bStatusCode, &dwSize))
			{
				if (bStatusCode == 0x00)
				{
					dwSize = 2;

					DWORD dwLen = 0;

					if (pTr->m_RcBuf.GetBulk(1, (PBYTE)&dwLen, &dwSize))
					{
						if (dwLen > 0)
						{
							dwSize = 1;

							if (pTr->m_RcBuf.GetBulk(3, pucInfo, &dwSize))
							{
								dwSize = 13;
								pTr->m_RcBuf.GetBulk(6, pPaddedUID, &dwSize);

								/*
								if ((*pucInfo == 0x83)&&(dwLen == 7))
								{
								dwSize = 4;

								pTr->m_RcBuf.GetBulk( 6, pUID, &dwSize );
								}
								if ((*pucInfo == 0x03)&&(dwLen == 10))
								{
								dwSize = 7;

								pTr->m_RcBuf.GetBulk( 6, pUID, &dwSize );
								}
								*/
							}

							dwSize = 2;

							pTr->m_RcBuf.GetBulk(4, pPiccCap, &dwSize);
						}
					}
				}
			}
		}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_VirtualCardSelect(IN LPVOID pReader,
IN DWORD dwKselFlashAddress,
IN PBYTE pPiccCap,
IN PBYTE pPaddedUID)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA3);
		pTr->m_TrBuf.AddByte(0x01);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwKselFlashAddress, 2);
		pTr->m_TrBuf.AddBulk(pPiccCap, 2);
		pTr->m_TrBuf.AddBulk(pPaddedUID, 13);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_VirtualCardDeselect(IN LPVOID pReader)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA3);
		pTr->m_TrBuf.AddByte(0x00);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

//=====================================================================

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_ProximityCheck(IN LPVOID pReader,
IN DWORD dwKproxFlashAddress)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0xA9);
		pTr->m_TrBuf.AddBulk((PBYTE)&dwKproxFlashAddress, 2);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

/*
CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifarePlus_WritePerso(	IN LPVOID pReader,
IN BYTE  ucValueType,
IN BYTE  ucSectorNumber,
IN BYTE  ucBlockNumber,
IN DWORD dwValueFlashAddress )
{
LONG Status = SCARD_S_SUCCESS;

if( pReader )
{
CTransfer* pTr = (CTransfer*)pReader;
pTr->m_TrBuf.Clear();
pTr->m_TrBuf.AddByte( 0xA8 );
pTr->m_TrBuf.AddByte( ucValueType );
pTr->m_TrBuf.AddByte( ucSectorNumber );
pTr->m_TrBuf.AddByte( ucBlockNumber );
pTr->m_TrBuf.AddBulk( (PBYTE)&dwValueFlashAddress, 2 );

Status = pTr->SingleTransceive();
}
else
{
Status = SCARD_E_READER_UNAVAILABLE;
}

return Status;
}
*/
