#include "StdAfx.h"

#include "Clscrfl.h"

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_HostCodeKey(IN     LPVOID pReader,
IN     LPBYTE pbUncoded,
OUT LPBYTE pbCoded)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x16);
		pTr->m_TrBuf.AddBulk(pbUncoded, 6);

		DWORD Size;
		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
		{
			Size = 12;
			if (!pTr->m_RcBuf.GetBulk(1, pbCoded, &Size))
				Status = SCARD_E_COMM_DATA_LOST;
		}
	}
	else
	{
		ZeroMemory(pbCoded, 12);
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_AuthKey(IN LPVOID pReader,
IN BYTE   bKeyType,
IN LPBYTE pbUID,
IN DWORD  dwSector,
IN LPBYTE pbCodedKey)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x18);
		pTr->m_TrBuf.AddByte(bKeyType);
		pTr->m_TrBuf.AddBulk(pbUID, 4);
		pTr->m_TrBuf.AddBulk(pbCodedKey, 12);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, 0));

		Status = pTr->SingleTransceive();

	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_AuthKeyDirect(IN LPVOID pReader,
IN BYTE   bKeyType,
IN LPBYTE pbUID,
IN DWORD  dwSector,
IN LPBYTE pbKey)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x14);
		pTr->m_TrBuf.AddByte(bKeyType);
		pTr->m_TrBuf.AddBulk(pbUID, 4);
		pTr->m_TrBuf.AddBulk(pbKey, 6);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, 0));

		Status = pTr->SingleTransceive();

	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_WriteKeyToE2(IN LPVOID pReader,
IN BYTE   bKeyType,
IN DWORD  dwE2RecNo,
IN LPBYTE pbUncoded)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x17);
		pTr->m_TrBuf.AddByte(bKeyType);
		pTr->m_TrBuf.AddByte((BYTE)dwE2RecNo & 0xff);
		pTr->m_TrBuf.AddBulk(pbUncoded, 6);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}


CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_AuthE2Ex(IN LPVOID pReader,
IN BYTE   bKeyType,
IN LPBYTE pbUID,
IN DWORD  dwE2RecNo,
IN DWORD  dwSector)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x15);
		pTr->m_TrBuf.AddByte(bKeyType);
		pTr->m_TrBuf.AddBulk(pbUID, 4);
		pTr->m_TrBuf.AddByte((BYTE)(dwE2RecNo & 0xff));
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, 0));

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_AuthE2(IN LPVOID pReader,
IN BYTE   bKeyType,
IN LPBYTE pbUID,
IN DWORD  dwSector)
{
	LONG Status = SCARD_S_SUCCESS;

	if (dwSector > 15)
		return SCARD_E_INVALID_PARAMETER;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x15);
		pTr->m_TrBuf.AddByte(bKeyType);
		pTr->m_TrBuf.AddBulk(pbUID, 4);
		pTr->m_TrBuf.AddByte((BYTE)dwSector & 0xff);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, 0));

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

/*
CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_AuthFlash( IN LPVOID pReader,
IN DWORD  dwAddress,
IN BYTE   bKeyType,
IN LPBYTE pbUID,
IN DWORD  dwSector )
{
LONG Status = SCARD_S_SUCCESS;

if( pReader )
{
CTransfer* pTr = (CTransfer*)pReader;
pTr->m_TrBuf.Clear();
pTr->m_TrBuf.AddByte( 0x26 );
pTr->m_TrBuf.AddBulk( (BYTE*)&dwAddress, 2 );
pTr->m_TrBuf.AddByte( bKeyType );
pTr->m_TrBuf.AddBulk( pbUID, 4 );
pTr->m_TrBuf.AddByte( (BYTE)dwSector & 0xff );

Status = pTr->SingleTransceive();
}
else
{
Status = SCARD_E_READER_UNAVAILABLE;
}

return Status;
}
*/

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Read(IN     LPVOID pReader,
IN     DWORD  dwSector,
IN     DWORD  dwBlock,
OUT LPBYTE pbRecvBuffer)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x19);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, dwBlock));

		DWORD Size;
		Status = pTr->SingleTransceive();

		if (Status == SCARD_S_SUCCESS)
			if (pTr->m_RcBuf.GetLength() != 17)
				Status = SCARD_E_COMM_DATA_LOST;
			else
			{
				Size = 16;
				if (!pTr->m_RcBuf.GetBulk(1, pbRecvBuffer, &Size))
					Status = SCARD_E_COMM_DATA_LOST;
			}
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}
	if (Status)
		ZeroMemory(pbRecvBuffer, 16);

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Write(IN LPVOID pReader,
IN DWORD  dwSector,
IN DWORD  dwBlock,
IN LPBYTE pbSendBuffer)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x1A);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, dwBlock));
		pTr->m_TrBuf.AddBulk(pbSendBuffer, 16);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

LONG MifareStandardValue(IN LPVOID pReader,
	IN BYTE   bOper,
	IN DWORD  dwSector,
	IN DWORD  dwSourceBlock,
	IN DWORD  dwValue,
	IN DWORD  dwTargetBlock)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x1B);
		pTr->m_TrBuf.AddByte(bOper);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, dwSourceBlock));
		pTr->m_TrBuf.AddBulk((unsigned char*)&dwValue, 4);
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, dwTargetBlock));

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Decrement(IN LPVOID pReader,
IN DWORD  dwSector,
IN DWORD  dwSourceBlock,
IN DWORD  dwValue,
IN DWORD  dwTargetBlock)
{
	return MifareStandardValue(pReader, PICC_DECREMENT,
		dwSector, dwSourceBlock, dwValue, dwTargetBlock);
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Increment(IN LPVOID pReader,
IN DWORD  dwSector,
IN DWORD  dwSourceBlock,
IN DWORD  dwValue,
IN DWORD  dwTargetBlock)
{
	return MifareStandardValue(pReader, PICC_INCREMENT,
		dwSector, dwSourceBlock, dwValue, dwTargetBlock);
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Restore(IN LPVOID pReader,
IN DWORD  dwSector,
IN DWORD  dwSourceBlock,
IN DWORD  dwTargetBlock)
{
	return MifareStandardValue(pReader, PICC_RESTORE,
		dwSector, dwSourceBlock, 0, dwTargetBlock);
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_EV1_PersonalizeUid(IN LPVOID pReader,
IN BYTE  ucUidMode)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x1C);
		pTr->m_TrBuf.AddByte(ucUidMode);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_EV1_SetLoadModulationType(IN LPVOID pReader,
IN BYTE  ucModType)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x26);
		pTr->m_TrBuf.AddByte(ucModType);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_CustomExchange(IN     LPVOID  pReader,
IN     WORD    wOption,
IN     LPCBYTE pbSendBuffer,
IN     DWORD   dwSendLength,
IN     BYTE    bTxLastBitsCount,
IN     DWORD   dwTimeout,
OUT LPBYTE  pbRecvBuffer,
IN OUT LPDWORD pdwRecvLength,
OUT LPBYTE  pRxLastBitsCount)
{
	LONG Status = SCARD_S_SUCCESS;
	DWORD BufSize = *pdwRecvLength;

	if (dwSendLength == 0)
	{
		*pdwRecvLength = 0;
		Status = SCARD_E_INVALID_PARAMETER;
	}
	else if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();

		pTr->m_TrBuf.AddByte(0x46);
		pTr->m_TrBuf.AddBulk((LPBYTE)&dwTimeout, 2);
		pTr->m_TrBuf.AddBulk((LPBYTE)&wOption, 2);
		pTr->m_TrBuf.AddByte(bTxLastBitsCount);
		pTr->m_TrBuf.AddBulk((LPBYTE)&dwSendLength, 2);
		pTr->m_TrBuf.AddBulk(pbSendBuffer, dwSendLength);

		Status = pTr->SingleTransceive();

		DWORD Size = pTr->m_RcBuf.GetLength();

		if (Size < 2)  // Length is absent
		{
			*pdwRecvLength = 0;
			if (Status == SCARD_S_SUCCESS)
				Status = SCARD_E_COMM_DATA_LOST;    // 0x8010002F
		}
		else
		{
			Size = 1;
			*pdwRecvLength = 0;
			if (!pTr->m_RcBuf.GetBulk(1, (LPBYTE)pRxLastBitsCount, &Size))
			{
				*pdwRecvLength = 0;
				if (Status == SCARD_S_SUCCESS)
					Status = SCARD_E_COMM_DATA_LOST;
			}

			Size = 2;
			if (!pTr->m_RcBuf.GetBulk(2, (LPBYTE)pdwRecvLength, &Size))
			{
				*pdwRecvLength = 0;
				if (Status == SCARD_S_SUCCESS)
					Status = SCARD_E_COMM_DATA_LOST;
			}

			if ((Size = *pdwRecvLength) > 0)
				if (!pTr->m_RcBuf.GetBulk(4, pbRecvBuffer, pdwRecvLength))
				{
					if (Status == SCARD_S_SUCCESS)
						Status = SCARD_E_COMM_DATA_LOST;
				}
		}
	}
	else
	{
		*pdwRecvLength = 0;
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

#ifdef CLSCRF_DES3

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Write3DES3(IN LPVOID pReader,
IN DWORD  dwKeyNumber,
IN DWORD  dwSector,
IN DWORD  dwBlock,
IN LPBYTE pbData)
{
	LONG Status = SCARD_S_SUCCESS;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x6b);
		pTr->m_TrBuf.AddByte(BYTE(dwKeyNumber & 0xff));
		pTr->m_TrBuf.AddByte(GetActualBlockAddress(dwSector, dwBlock));
		pTr->m_TrBuf.AddBulk(pbData, 16);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

#endif //CLSCRF_DES3

/*
CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_WriteFromFlash(	IN LPVOID pReader,
IN DWORD  dwFlashAddr,
IN DWORD  dwSector,
IN DWORD  dwBlock )
{
LONG Status = SCARD_S_SUCCESS;

if( pReader )
{
CTransfer* pTr = (CTransfer*)pReader;
pTr->m_TrBuf.Clear();
pTr->m_TrBuf.AddByte( 0x63 );
pTr->m_TrBuf.AddBulk( (PBYTE)&dwFlashAddr, 2 );
pTr->m_TrBuf.AddByte( GetActualBlockAddress( dwSector, dwBlock ) );

Status = pTr->SingleTransceive();
}
else
{
Status = SCARD_E_READER_UNAVAILABLE;
}

return Status;
}
*/

#ifdef CLSCRF_DES3

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_WriteKeysToE2_3DES3(IN LPVOID pReader,
IN DWORD  dwKeyNumber,
IN DWORD  dwSector,
IN LPBYTE pbUncodedKeys)
{
	LONG Status = SCARD_S_SUCCESS;

	if (dwSector > 15)
		return SCARD_E_INVALID_PARAMETER;

	if (pReader)
	{
		CTransfer* pTr = (CTransfer*)pReader;
		pTr->m_TrBuf.Clear();
		pTr->m_TrBuf.AddByte(0x6d);
		pTr->m_TrBuf.AddByte(BYTE(dwKeyNumber & 0xff));
		pTr->m_TrBuf.AddByte((BYTE)dwSector & 0xff);
		pTr->m_TrBuf.AddBulk(pbUncodedKeys, 16);

		Status = pTr->SingleTransceive();
	}
	else
	{
		Status = SCARD_E_READER_UNAVAILABLE;
	}

	return Status;
}

#endif //CLSCRF_DES3

/*
CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Activate_TCL_RATS( IN LPVOID pReader,
IN BYTE  ucCID,
OUT LPBYTE  pbDSDR,
OUT LPBYTE  pbATS )
{
LONG Status = SCARD_S_SUCCESS;

DWORD dwTimeout = 0;

unsigned char b, DRI, DSI, DSDR = 0;

const int BUF_SIZE = 300;
BYTE SendBuffer[BUF_SIZE];
BYTE RecvBuffer[BUF_SIZE];
DWORD SendLength;
DWORD RecvLength = BUF_SIZE;
//    DWORD Timeout = 6800;

SendBuffer[0] = 0xe0;                           // RATS
SendBuffer[1] = (cPCD_FSDI << 4) | ucCID;// Parameter codes FSDI & CID
SendLength = 2;

Status = CLSCRF_DirectIO_Card( pReader,
SendBuffer, SendLength,
dwTimeout,//Timeout,
RecvBuffer, &RecvLength );

if( Status == 0 )
{
if( RecvBuffer[0] > 0 )
memcpy( pbATS, RecvBuffer, RecvBuffer[0] );
if( RecvBuffer[0] > 1 )
{
b = RecvBuffer[1];        //(Data[5])     // Format byte T0
if( b & 0x10 )              // TA(1) is transmitted
{
DSDR = RecvBuffer[2];    //(Data[6])
*pbDSDR = DSDR;
}
}
}

return Status;
}

CLSCRF_API LONG CLSCRF_DECL
CLSCRF_MifareStandard_Activate_TCL_PPS( IN LPVOID pReader,
IN BYTE  ucCID,
IN BYTE  ucBaudrate )
{
LONG Status = SCARD_S_SUCCESS;

DWORD dwTimeout = 0;

const int BUF_SIZE = 300;
BYTE SendBuffer[BUF_SIZE];
BYTE RecvBuffer[BUF_SIZE];
DWORD SendLength;
DWORD RecvLength = BUF_SIZE;
//    DWORD Timeout = 6800;

SendBuffer[0] = 0xd0 | ucCID;                // PPSS
SendBuffer[1] = 0x11;                               // PPS0: PPS1 is presence
SendBuffer[2] = ( ucBaudrate << 2 ) | ucBaudrate; // PPS1:
SendLength = 3;

Status = CLSCRF_DirectIO_Card( pReader,
SendBuffer, SendLength,
dwTimeout,//Timeout,
RecvBuffer, &RecvLength );

if( !(Status == 0 && RecvLength == 1 && (RecvBuffer[0] & 0xf0) == 0xd0) )
return -1;

Status = CLSCRF_Mfrc_Set_Rf_Mode( pReader,
( ucBaudrate << 2 ) | ucBaudrate );

return Status;
}

*/
