#ifndef __oststd__
#define	__oststd__

/* Typdefinitionen */

typedef unsigned char uc;
typedef uc perm64[65];
typedef uc uc8[8];


/* Funktions-Return-Codes */

#ifndef OK
#define OK			0
#endif


#ifndef NOT_OK
#define NOT_OK		       -1	
#endif

#ifndef TRUE
#define TRUE    		1
#endif
#ifndef FALSE
#define FALSE   		0
#endif

#define OSTstdMathMax_c 		256

/* crc */

/* Achtung:
 *
 * Beim CCITT-Polynom werden die Bits von LSB-first kodiert:
 *      00 01 02 03 | 04 05 06 07  | 08 09 10 11 | 12 13 14 15
 *
 * Beim GAO-Polynom werden die Bits in folgender Reihenfolge kodiert:
 *      00 01 02 03 | 07 06 05 04  | 11 10 09 08 | 15 14 13 12
 *
 * Die Stelle x^16 ist immer gesetzt.
 */

#define OSTstdCrcCCITTAlg_c    0
#define OSTstdCrcGAOAlg_c      1

/* Polynom: x^16 + x^12 + x^5 + x^0 */
#define OSTstdCrcCCITTPol_c    0x8408
#define OSTstdCrcGAOPol_c      0x1021

/* Polynom: x^16 + x^15 + x^2 + x^0 */
#define OSTstdCrcCCITTPol2_c  0xA001
#define OSTstdCrcGAOPol2_c    0x5008


#define OSTstdCrcCCITT_c        OSTstdCrcCCITTAlg_c, OSTstdCrcCCITTPol_c
#define OSTstdCrcGAO_c          OSTstdCrcGAOAlg_c,   OSTstdCrcGAOPol_c


/******************************************************************************
*
*	void OSTstdDesKey(uc8 key)
*
*	Sets DES-Keys for further de- and encrypting function calls
*
*	Parameters:
*
*		key		-	8 Byte DES-Key
*
******************************************************************************/
void OSTstdDesKey(uc8 key);

/******************************************************************************
*
*	void OUSTstdDesEncrypt(uc8 kl, uc8 R)
*
*	DES-Encryption
*
*	Parameters:
*
*		kl		-	8 Byte data to encrypt
*		R		-	8 Byte encrypted data
*
******************************************************************************/
void OSTstdDesEncrypt(uc8 kl,uc8 R);




/******************************************************************************
*
*	void OSTstdDesDecrypt(uc8 kl, uc8 R)
*
*	DES-Decryption
*
*	Parameters:
*
*		kl		-	8 Byte data to decrypt
*		R		-	8 Byte decrypted data
*
******************************************************************************/
void OSTstdDesDecrypt(uc8 kl,uc8 R);

/*----------------------------------------------------------------------------*/
/*                     DES CBC De&En cryption Functions        				  */
/*----------------------------------------------------------------------------*/

void des_cbc_decrypt(unsigned char *key,unsigned char *source,
					 unsigned char *destination,int l,unsigned char *icvin=NULL);
void des_cbc_encrypt(unsigned char *key,unsigned char *source,
					 unsigned char *destination,int l,unsigned char *icvin=NULL,unsigned char *mac_key=NULL);

void des_3cbc_decrypt(unsigned char *ks1,unsigned char *ks2,unsigned char *source,
					 unsigned char *destination,int length,unsigned char *icvin1=NULL,unsigned char *icvin2=NULL,unsigned char *iv_result=NULL);
void des_3cbc_encrypt(unsigned char *ks1,unsigned char *ks2,unsigned char *source,
					 unsigned char *destination,int length,unsigned char *icvin1=NULL,unsigned char *icvin2=NULL,unsigned char *iv_result=NULL,unsigned char *mac_key=NULL);
void des_3cbc_encipher_by_decrypt(unsigned char *ks1,unsigned char *ks2,unsigned char *source,
					 unsigned char *destination,int length,unsigned char *icvin1=NULL,unsigned char *icvin2=NULL,unsigned char *iv_result=NULL);

void des_3k3cbc_decrypt(unsigned char *ks1,unsigned char *ks2,unsigned char *ks3,unsigned char *source,
					 unsigned char *destination,int length,unsigned char *icvin1=NULL,unsigned char *icvin2=NULL,unsigned char *iv_result=NULL);
void des_3k3cbc_encrypt(unsigned char *ks1,unsigned char *ks2,unsigned char *ks3,unsigned char *source,
					 unsigned char *destination,int length,unsigned char *icvin1=NULL,unsigned char *icvin2=NULL,unsigned char *iv_result=NULL,unsigned char *mac_key=NULL);
#endif

