#include "../StdAfx.h"
#include "NdefMessage.h"

NdefMessage::NdefMessage(void)
{
    _recordCount = 0;
}

NdefMessage::NdefMessage(const byte * data, const int numBytes)
{
    #ifdef NDEF_DEBUG
    Serial.print(F("Decoding "));Serial.print(numBytes);Serial.println(F(" bytes"));
    PrintHexChar(data, numBytes);
    //DumpHex(data, numBytes, 16);
    #endif

    _recordCount = 0;

    int index = 0;

    while (index <= numBytes)
    {

        // decode tnf - first byte is tnf with bit flags
        // see the NFDEF spec for more info
        byte tnf_byte = data[index];
        bool mb = (tnf_byte & 0x80) != 0;
        bool me = (tnf_byte & 0x40) != 0;
        bool cf = (tnf_byte & 0x20) != 0;
        bool sr = (tnf_byte & 0x10) != 0;
        bool il = (tnf_byte & 0x8) != 0;
        byte tnf = (tnf_byte & 0x7);

        NdefRecord* record = new NdefRecord();
        record->setTnf(tnf);

        index++;
        int typeLength = data[index];

        int payloadLength = 0;
        if (sr)
        {
            index++;
            payloadLength = data[index];
        }
        else
        {
            payloadLength =
		((0xFF & data[++index]) << 24)
		| ((0xFF & data[++index]) << 16)
		| ((0xFF & data[++index]) << 8)
		| (0xFF & data[++index]);
        }

        int idLength = 0;
        if (il)
        {
            index++;
            idLength = data[index];
        }

        index++;
        record->setType(&data[index], typeLength);
        index += typeLength;

        if (il)
        {
            record->setId(&data[index], idLength);
            index += idLength;
        }

        record->setPayload(&data[index], payloadLength);
        index += payloadLength;

        addRecord(record);

        if (me) break; // last message
    }

}

NdefMessage::NdefMessage(NdefMessage* rhs)
{

    _recordCount = rhs->_recordCount;
    for (int i = 0; i < _recordCount; i++)
    {
        _records[i] = rhs->_records[i];
    }

}

NdefMessage::~NdefMessage()
{
	for (int i = 0; i < _recordCount; i++)
	{
		// TODO Dave: is this the right way to delete existing records?
		try
		{
			free(_records[i]);// = NdefRecord();
			_records[i] = NULL;
		}
		catch (int e)
		{
		}

	}

	_recordCount = 0;
}

NdefMessage* NdefMessage::operator=(NdefMessage* rhs)
{

    if (this != rhs)
    {

        // delete existing records
        for (int i = 0; i < _recordCount; i++)
        {
            // TODO Dave: is this the right way to delete existing records?
			free(_records[i]);// = NdefRecord();
			_records[i] = NULL;
        }

        _recordCount = rhs->_recordCount;
        for (int i = 0; i < _recordCount; i++)
        {
            _records[i] = rhs->_records[i];
        }
    }
    return this;
}

unsigned int NdefMessage::getRecordCount()
{
    return _recordCount;
}

int NdefMessage::getEncodedSize()
{
    int size = 0;
    for (int i = 0; i < _recordCount; i++)
    {
        size += _records[i]->getEncodedSize();
    }
    return size;
}

// TODO change this to return uint8_t*
void NdefMessage::encode(uint8_t* data)
{
    // assert sizeof(data) >= getEncodedSize()
    uint8_t* data_ptr = &data[0];

    for (int i = 0; i < _recordCount; i++)
    {
        _records[i]->encode(data_ptr, i == 0, (i + 1) == _recordCount);
        // TODO can NdefRecord.encode return the record size?
        data_ptr += _records[i]->getEncodedSize();
    }

}

boolean NdefMessage::addRecord(NdefRecord* record)
{

    if (_recordCount < MAX_NDEF_RECORDS)
    {
        _records[_recordCount] = record;
        _recordCount++;
        return true;
    }
    else
    {
        //Serial.println(F("WARNING: Too many records. Increase MAX_NDEF_RECORDS."));
        return false;
    }
}

//void NdefMessage::addMimeMediaRecord(CString mimeType, CString payload)
void NdefMessage::addMimeMediaRecord(std::string mimeType, std::string payload)
{

	//byte payloadBytes[payload.GetLength() + 1];
    //payload.getBytes(payloadBytes, sizeof(payloadBytes));

	//addMimeMediaRecord(mimeType, payloadBytes, payload.GetLength());
	addMimeMediaRecord(mimeType, (byte*)payload.c_str(), payload.length());
	//payload.ReleaseBuffer();
}

//void NdefMessage::addMimeMediaRecord(CString mimeType, uint8_t* payload, int payloadLength)
void NdefMessage::addMimeMediaRecord(std::string mimeType, uint8_t* payload, int payloadLength)
{
    NdefRecord* r = new NdefRecord();
    r->setTnf(TNF_MIME_MEDIA);

	//byte type[mimeType.GetLength() + 1];
	//byte type[256] = { 0 };
    //mimeType.getBytes(type, sizeof(type));
	//memcpy(type, mimeType.GetBuffer(), min(mimeType.GetLength(), sizeof(type)));
	r->setType((byte*)mimeType.c_str(), mimeType.length());
	//mimeType.ReleaseBuffer();

    r->setPayload(payload, payloadLength);

    addRecord(r);
}

//void NdefMessage::addTextRecord(CString text)
void NdefMessage::addTextRecord(std::string text)
{
    addTextRecord(text, "en");
}

//void NdefMessage::addTextRecord(CString text, CString encoding)
void NdefMessage::addTextRecord(std::string text, std::string encoding)
{
    NdefRecord* r = new NdefRecord();
    r->setTnf(TNF_WELL_KNOWN);

    uint8_t RTD_TEXT[1] = { 0x54 }; // TODO this should be a constant or preprocessor
    r->setType(RTD_TEXT, sizeof(RTD_TEXT));

    // X is a placeholder for encoding length
    // TODO is it more efficient to build w/o string concatenation?
    //CString payloadString = "X" + encoding + text;
    std::string payloadString = "X" + encoding + text;

	byte* payload = new byte[payloadString.length() + 1];
	memset(payload, 0, payloadString.length() + 1);
	//payloadString.getBytes(payload, sizeof(payload));
	memcpy(payload, payloadString.c_str(), payloadString.length());
	//payloadString.ReleaseBuffer();

    // replace X with the real encoding length
	payload[0] = 0x00 + encoding.length(); // (0x00 + ... for UTF-8, 0x80 + ... for UTF-16)

	r->setPayload(payload, payloadString.length());

	free(payload);

    addRecord(r);
}

//void NdefMessage::addUriRecord(CString uri)
void NdefMessage::addUriRecord(std::string uri)
{
    NdefRecord* r = new NdefRecord();
    r->setTnf(TNF_WELL_KNOWN);

    uint8_t RTD_URI[1] = { 0x55 }; // TODO this should be a constant or preprocessor
    r->setType(RTD_URI, sizeof(RTD_URI));

    // X is a placeholder for identifier code
    //CString payloadString = "X" + uri;
    std::string payloadString = "X" + uri;

    byte* payload = new byte[payloadString.length() + 1];
	memset(payload, 0, payloadString.length() + 1);
    //payloadString.getBytes(payload, sizeof(payload));
	memcpy(payload, payloadString.c_str(), payloadString.length());
	//payloadString.ReleaseBuffer();
	// add identifier code 0x0, meaning no prefix substitution
    payload[0] = 0x0;

	r->setPayload(payload, payloadString.length());

	free(payload);

    addRecord(r);
    //delete(r);
}

void NdefMessage::addEmptyRecord()
{
    NdefRecord* r = new NdefRecord();
    r->setTnf(TNF_EMPTY);
    addRecord(r);
    //delete(r);
}

NdefRecord* NdefMessage::getRecord(int index)
{
    if (index > -1 && index < _recordCount)
    {
        return _records[index];
    }
    else
    {
        return new NdefRecord(); // would rather return NULL
    }
}

NdefRecord* NdefMessage::operator[](int index)
{
    return getRecord(index);
}

void NdefMessage::print()
{
    //Serial.print(F("\nNDEF Message "));Serial.print(_recordCount);Serial.print(F(" record"));
    //_recordCount == 1 ? Serial.print(", ") : Serial.print("s, ");
    //Serial.print(getEncodedSize());Serial.println(F(" bytes"));

    int i;
    for (i = 0; i < _recordCount; i++)
    {
         _records[i]->print();
    }
}
