#include "StdAfx.h"
#include "aes_128.h"

//#include <stdlib.h>

//   ,   
#define Nb 4
//   ,     
#define Nk 4
//  
#define Nr 10

typedef unsigned char uint8;
typedef unsigned int uint32;

//  -      
static void KeyExpansion(unsigned char key[], uint8 w[4][Nb * (Nr + 1)]);
//      w[start..start + Nb - 1]
static void AddRoundKey(uint8 state[][Nb], uint8 w[4][Nb * (Nr + 1)], int start);
//       S_box
static void SubBytes(uint8 state[][Nb]);
//     (i-  
// i )
static void ShiftRows(uint8 state[][Nb]);
// 
static void MixColumns(uint8 state[][Nb]);

//     (i-  
// i )
static void InvShiftRows(uint8 state[][Nb]);
//       inv_S_box
static void InvSubBytes(uint8 state[][Nb]);
// 
static void InvMixColumns(uint8 state[][Nb]);

//  input[]   inputLength  key[16]   bc    iv[16],
//   ISO 10126,    output[],     
unsigned int aes_128_encrypt_cbc_mod(unsigned char input[], unsigned int inputLength, unsigned char key[], unsigned char iv[], 
                                 unsigned char output[])
{
    unsigned char curData[16], curIV[16];
    unsigned int i, j, k;

    if (inputLength == 0)
        return 0;

    //     
    i = 0;
    while (i <= 15 && i <= inputLength - 1)
        curData[i++] = input[i];
    if (i <= 15)
    {
        for (j = i; j <= 14; j++)
            curData[j] = (unsigned char)rand();
        curData[15] = (unsigned char)(15 - i + 1);
    }

    for (i = 0; i <= 15; i++)
        curIV[i] = iv[i];

    i = 0;
    while (i < inputLength)
    {
        for (j = 0; j <= 15; j++)
            curData[j] ^= curIV[j];
        aes_128_encrypt16(curData, key, output + i);

        for (j = 0; j <= 15; j++)
            curIV[j] = output[i + j];

        i += 16;
        if (i < inputLength)
        {
            //     
            j = i;
            while (j <= i + 15 && j <= inputLength - 1)
                curData[j - i] = input[j++];
            if (j <= i + 15)
            {
                for (k = j; k <= i + 14; k++)
                    curData[k - i] = (unsigned char)rand();
                curData[15] = (unsigned char)(15 - (j - i) + 1);
            }
        }
    }

    if (i == inputLength)
    {
        for (j = 0; j <= 14; j++)
            curData[j] = (unsigned char)rand();
        curData[15] = 16;
        for (j = 0; j <= 15; j++)
            curData[j] ^= curIV[j];

        aes_128_encrypt16(curData, key, output + i);
        i += 16;
    }

    return i;
}

unsigned int aes_128_encrypt_cbc(unsigned char input[], unsigned int inputLength, unsigned char key[], unsigned char iv[], 
                                 unsigned char output[], unsigned char *iv_result, unsigned char *mac_key)
{
	unsigned char curData[16] = {0}, curIV[16] = {0};
    unsigned int i;

    if (inputLength == 0)
        return 0;

    //     
	memcpy(curData, input, (inputLength>16)?16:inputLength);

	memcpy(curIV, iv, 16);

    i = 0;
    while (i < inputLength)
    {
		xor_128(curData, curIV, curData);

        if ((mac_key)&&(i+16 == inputLength))
			xor_128(curData, mac_key, curData);

        aes_128_encrypt16(curData, key, output + i);

		memcpy(curIV, &output[i], 16);

        i += 16;
        if (i < inputLength)
			memcpy(curData, &input[i], (inputLength>16+i)?16:(inputLength-i));
    }

	if (iv_result)
		memcpy(iv_result, curIV, 16);

/*
    if (i == inputLength)
    {
        for (j = 0; j <= 15; j++)
            curData[j] ^= curIV[j];

        if (mac_key)
		for (j = 0; j <= 15; j++)
            curData[j] ^= mac_key[j];

        aes_128_encrypt16(curData, key, output + i);
        i += 16;
    }
*/
    return i;
}

unsigned int aes_192_encrypt_cbc(unsigned char input[], unsigned int inputLength, unsigned char key[], unsigned char iv[], 
                                 unsigned char output[], unsigned char *iv_result, unsigned char *mac_key)
{
	unsigned char curData[24] = {0}, curIV[24] = {0};
    unsigned int i;

    if (inputLength == 0)
        return 0;

    //     
	memcpy(curData, input, (inputLength>24)?24:inputLength);

	memcpy(curIV, iv, 24);

    i = 0;
    while (i < inputLength)
    {
		xor_192(curData, curIV, curData);

        if ((mac_key)&&(i+24 == inputLength))
			xor_192(curData, mac_key, curData);

        aes_192_encrypt24(curData, key, output + i);

		memcpy(curIV, &output[i], 24);

        i += 24;
        if (i < inputLength)
			memcpy(curData, &input[i], (inputLength>24+i)?24:(inputLength-i));
    }

	if (iv_result)
		memcpy(iv_result, curIV, 24);

    return i;
}

//  input[]   inputLength  key[16]   cbc    iv[16],
//   ISO 10126,    output[],     
unsigned int aes_128_decrypt_cbc_mod(unsigned char input[], unsigned int inputLength, unsigned char key[], unsigned char iv[], 
                                 unsigned char output[])
{
	unsigned char curData[16], curIV[16];
    unsigned int i = 0, l = 0;
    int k, j;

    if (inputLength == 0 || inputLength % 16 != 0)
        return 0;

    for (j = 0; j <= 15; j++)
        curIV[j] = iv[j];

    while (i < inputLength)
    {
        aes_128_decrypt16(input + i, key, curData);
        for (j = 0; j <= 15; j++)
            curData[j] ^= curIV[j];

        if (i + 16 == inputLength)
        {
            k = 16 - curData[15];
            if (k < 0 || k > 16)
                return 0;
            else
            if (k != 0)
                for (j = 0; j <= k - 1; j++)
                    output[i + j] = curData[j];
            l = i + k;
        }
        else
        {
            for (j = 0; j <= 15; j++)
                output[i + j] = curData[j];
        }

        if (i < inputLength)
            for (j = 0; j <= 15; j++)
                curIV[j] = input[j + i];

        i += 16;
    }

    return l;
}

unsigned int aes_128_decrypt_cbc(unsigned char input[], unsigned int inputLength, unsigned char key[], unsigned char iv[], 
                                 unsigned char output[], unsigned char *iv_result, unsigned char *mac_key)
{
	unsigned char curData[16] = {0}, curIV[16] = {0};
    unsigned int i = 0, l = 0;

    if (inputLength == 0 || inputLength % 16 != 0)
        return 0;

	memcpy(curIV, iv, 16);

    while (i < inputLength)
    {
        aes_128_decrypt16(input + i, key, curData);

		xor_128(curData, curIV, curData);

        if (mac_key)
			xor_128(curData, mac_key, curData);

		memcpy(&output[i], curData, 16);

		memcpy(curIV, &input[i], 16);

		i += 16;
    }

	if (iv_result)
		memcpy(iv_result, &input[i-16], 16);

    return l;
}

//  input[16]  key[16],
//   output[16]
void aes_128_encrypt16(unsigned char input[], unsigned char key[], unsigned char output[])
{
    uint8 state[4][Nb];
    uint8 w[4][Nb * (Nr + 1)];
    int r, c, round;

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            state[r][c] = input[r + 4 * c];

    KeyExpansion(key, w);
    AddRoundKey(state, w, 0);

    for (round = 1; round <= Nr - 1; round++)
    {
        SubBytes(state);
        ShiftRows(state);
        MixColumns(state);
        AddRoundKey(state, w, round * Nb);
    }

    SubBytes(state);
    ShiftRows(state);
    AddRoundKey(state, w, Nr * Nb);

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            output[r + 4 * c] = state[r][c];
}

//  input[24]  key[24],
//   output[24]
void aes_192_encrypt24(unsigned char input[], unsigned char key[], unsigned char output[])
{
    uint8 state[4][Nb];
    uint8 w[4][Nb * (Nr + 1)];
    int r, c, round;

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            state[r][c] = input[r + 4 * c];

    KeyExpansion(key, w);
    AddRoundKey(state, w, 0);

    for (round = 1; round <= Nr - 1; round++)
    {
        SubBytes(state);
        ShiftRows(state);
        MixColumns(state);
        AddRoundKey(state, w, round * Nb);
    }

    SubBytes(state);
    ShiftRows(state);
    AddRoundKey(state, w, Nr * Nb);

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            output[r + 4 * c] = state[r][c];
}

//  input[16]  key[16],
//   output[16]
void aes_128_decrypt16(unsigned char input[], unsigned char key[], unsigned char output[])
{
    uint8 state[4][Nb];
    uint8 w[4][Nb * (Nr + 1)];
    int r, c, round;

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            state[r][c] = input[r + 4 * c];

    KeyExpansion(key, w);
    AddRoundKey(state, w, Nr * Nb);

    for (round = Nr - 1; round >= 1; round--)
    {
        InvShiftRows(state);
        InvSubBytes(state);
        AddRoundKey(state, w, round * Nb);
        InvMixColumns(state);
    }

    InvShiftRows(state);
    InvSubBytes(state);
    AddRoundKey(state, w, 0);

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            output[r + 4 * c] = state[r][c];
}

//  input[24]  key[24],
//   output[24]
void aes_192_decrypt24(unsigned char input[], unsigned char key[], unsigned char output[])
{
    uint8 state[4][Nb];
    uint8 w[4][Nb * (Nr + 1)];
    int r, c, round;

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            state[r][c] = input[r + 4 * c];

    KeyExpansion(key, w);
    AddRoundKey(state, w, Nr * Nb);

    for (round = Nr - 1; round >= 1; round--)
    {
        InvShiftRows(state);
        InvSubBytes(state);
        AddRoundKey(state, w, round * Nb);
        InvMixColumns(state);
    }

    InvShiftRows(state);
    InvSubBytes(state);
    AddRoundKey(state, w, 0);

    for (r = 0; r <= 3; r++)
        for (c = 0; c <= Nb - 1; c++)
            output[r + 4 * c] = state[r][c];
}

//  input[32]  key[16]   cbc    iv[16],
//   output[32]
void aes_128_encrypt32_cbc(unsigned char input[], unsigned char key[], unsigned char iv[], unsigned char output[])
{
    unsigned char temp[16];
    int i;

    for (i = 0; i <= 15; i++)
        temp[i] = input[i] ^ iv[i];
    aes_128_encrypt16(temp, key, output);

    for (i = 0; i <= 15; i++)
        temp[i] = input[i + 16] ^ output[i];
    aes_128_encrypt16(temp, key, output + 16);
}

//  input[32]  key[16]   cbc    iv[16],
//   output[32]
void aes_128_decrypt32_cbc(unsigned char input[], unsigned char key[], unsigned char iv[], unsigned char output[])
{
    int i;

    aes_128_decrypt16(input, key, output);
    for (i = 0; i <= 15; i++)
        output[i] ^= iv[i];

    aes_128_decrypt16(input + 16, key, output + 16);
    for (i = 0; i <= 15; i++)
        output[i + 16] ^= input[i];
}

//,      
static const uint8 S_box[256] = {
0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76,
0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0,
0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15,
0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75,
0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84,
0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b, 0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf,
0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8,
0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2,
0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73,
0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb,
0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08,
0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a,
0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e,
0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf,
0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16};

//    
static const uint8 Rcon[(Nb * (Nr + 1) - 1) / Nk + 1] = {0x00,
0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36};

//      S_box
static void SubWord(uint8[]);
//  : [a0,a1,a2,a3]->[a1,a2,a3,a0] 
static void RotWord(uint8[]);

uint32 word(uint32 x0, uint32 x1, uint32 x2, uint32 x3) 
{
    return (x0 << 24) + (x1 << 16) + (x2 << 8) + (x3 << 0);
}

//  -      
void KeyExpansion(unsigned char key[], uint8 w[4][Nb * (Nr + 1)])
{
    int i, j;
    uint8 temp[4];

    for (i = 0; i <= Nk - 1; i++)
        for (j = 0; j <= 3; j++)
            w[j][i] = key[4 * i + j];

    for (i = Nk; i <= Nb * (Nr + 1) - 1; i++)
    {
        for (j = 0; j <= 3; j++)
            temp[j] = w[j][i - 1];
        if (i % Nk == 0)
        {
            RotWord(temp);
            SubWord(temp);
            temp[0] = temp[0] ^ Rcon[i / Nk];
        }
        else
        if (Nk > 6 && i % Nk == 4)
        {
            SubWord(temp);
        }
        for (j = 0; j <= 3; j++)
            w[j][i] = w[j][i - Nk] ^ temp[j];
    }
}

//      S_box
void SubWord(uint8 v[])
{
    int i;

    for (i = 0; i <= 3; i++)
        v[i] = S_box[v[i]];
}

//  : [a0,a1,a2,a3]->[a1,a2,a3,a0] 
void RotWord(uint8 v[])
{
    uint8 temp = v[0];
    
    v[0] = v[1];
    v[1] = v[2];
    v[2] = v[3];
    v[3] = temp;
}

//      w[start..start + Nb - 1]
void AddRoundKey(uint8 state[][Nb], uint8 w[4][Nb * (Nr + 1)], int start)
{
    int i, j;

    for (i = 0; i <= Nb - 1; i++)
        for (j = 0; j <= 3; j++)
            state[j][i] ^= w[j][start + i];
}

//       S_box
void SubBytes(uint8 state[][Nb])
{
    int i, j;

    for (i = 0; i <= Nb - 1; i++)
        for (j = 0; j <= 3; j++)
            state[j][i] = S_box[state[j][i]];
}

//     (i-  
// i )
void ShiftRows(uint8 state[][Nb])
{
    uint8 temp[Nb];
    int i, j, n;

    for (i = 1; i <= 3; i++)
    {
        for (j = 0; j <= Nb - 1; j++)
            temp[j] = state[i][j];
        for (j = 0; j <= Nb - 1; j++)
        {
            n = (j - i < 0) ? (j - i + Nb) : (j - i);
            state[i][n] = temp[j];
        }
    }
}

//   x  2   GF(2^8)
static const uint8 X_mul_2[256] = { 
0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x14, 0x16, 0x18, 0x1a, 0x1c, 0x1e,
0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e, 0x30, 0x32, 0x34, 0x36, 0x38, 0x3a, 0x3c, 0x3e,
0x40, 0x42, 0x44, 0x46, 0x48, 0x4a, 0x4c, 0x4e, 0x50, 0x52, 0x54, 0x56, 0x58, 0x5a, 0x5c, 0x5e,
0x60, 0x62, 0x64, 0x66, 0x68, 0x6a, 0x6c, 0x6e, 0x70, 0x72, 0x74, 0x76, 0x78, 0x7a, 0x7c, 0x7e,
0x80, 0x82, 0x84, 0x86, 0x88, 0x8a, 0x8c, 0x8e, 0x90, 0x92, 0x94, 0x96, 0x98, 0x9a, 0x9c, 0x9e,
0xa0, 0xa2, 0xa4, 0xa6, 0xa8, 0xaa, 0xac, 0xae, 0xb0, 0xb2, 0xb4, 0xb6, 0xb8, 0xba, 0xbc, 0xbe,
0xc0, 0xc2, 0xc4, 0xc6, 0xc8, 0xca, 0xcc, 0xce, 0xd0, 0xd2, 0xd4, 0xd6, 0xd8, 0xda, 0xdc, 0xde,
0xe0, 0xe2, 0xe4, 0xe6, 0xe8, 0xea, 0xec, 0xee, 0xf0, 0xf2, 0xf4, 0xf6, 0xf8, 0xfa, 0xfc, 0xfe,
0x1b, 0x19, 0x1f, 0x1d, 0x13, 0x11, 0x17, 0x15, 0x0b, 0x09, 0x0f, 0x0d, 0x03, 0x01, 0x07, 0x05,
0x3b, 0x39, 0x3f, 0x3d, 0x33, 0x31, 0x37, 0x35, 0x2b, 0x29, 0x2f, 0x2d, 0x23, 0x21, 0x27, 0x25,
0x5b, 0x59, 0x5f, 0x5d, 0x53, 0x51, 0x57, 0x55, 0x4b, 0x49, 0x4f, 0x4d, 0x43, 0x41, 0x47, 0x45,
0x7b, 0x79, 0x7f, 0x7d, 0x73, 0x71, 0x77, 0x75, 0x6b, 0x69, 0x6f, 0x6d, 0x63, 0x61, 0x67, 0x65,
0x9b, 0x99, 0x9f, 0x9d, 0x93, 0x91, 0x97, 0x95, 0x8b, 0x89, 0x8f, 0x8d, 0x83, 0x81, 0x87, 0x85,
0xbb, 0xb9, 0xbf, 0xbd, 0xb3, 0xb1, 0xb7, 0xb5, 0xab, 0xa9, 0xaf, 0xad, 0xa3, 0xa1, 0xa7, 0xa5,
0xdb, 0xd9, 0xdf, 0xdd, 0xd3, 0xd1, 0xd7, 0xd5, 0xcb, 0xc9, 0xcf, 0xcd, 0xc3, 0xc1, 0xc7, 0xc5,
0xfb, 0xf9, 0xff, 0xfd, 0xf3, 0xf1, 0xf7, 0xf5, 0xeb, 0xe9, 0xef, 0xed, 0xe3, 0xe1, 0xe7, 0xe5};

//   x  3   GF(2^8)
static const uint8 X_mul_3[256] = { 
0x00, 0x03, 0x06, 0x05, 0x0c, 0x0f, 0x0a, 0x09, 0x18, 0x1b, 0x1e, 0x1d, 0x14, 0x17, 0x12, 0x11,
0x30, 0x33, 0x36, 0x35, 0x3c, 0x3f, 0x3a, 0x39, 0x28, 0x2b, 0x2e, 0x2d, 0x24, 0x27, 0x22, 0x21,
0x60, 0x63, 0x66, 0x65, 0x6c, 0x6f, 0x6a, 0x69, 0x78, 0x7b, 0x7e, 0x7d, 0x74, 0x77, 0x72, 0x71,
0x50, 0x53, 0x56, 0x55, 0x5c, 0x5f, 0x5a, 0x59, 0x48, 0x4b, 0x4e, 0x4d, 0x44, 0x47, 0x42, 0x41,
0xc0, 0xc3, 0xc6, 0xc5, 0xcc, 0xcf, 0xca, 0xc9, 0xd8, 0xdb, 0xde, 0xdd, 0xd4, 0xd7, 0xd2, 0xd1,
0xf0, 0xf3, 0xf6, 0xf5, 0xfc, 0xff, 0xfa, 0xf9, 0xe8, 0xeb, 0xee, 0xed, 0xe4, 0xe7, 0xe2, 0xe1,
0xa0, 0xa3, 0xa6, 0xa5, 0xac, 0xaf, 0xaa, 0xa9, 0xb8, 0xbb, 0xbe, 0xbd, 0xb4, 0xb7, 0xb2, 0xb1,
0x90, 0x93, 0x96, 0x95, 0x9c, 0x9f, 0x9a, 0x99, 0x88, 0x8b, 0x8e, 0x8d, 0x84, 0x87, 0x82, 0x81,
0x9b, 0x98, 0x9d, 0x9e, 0x97, 0x94, 0x91, 0x92, 0x83, 0x80, 0x85, 0x86, 0x8f, 0x8c, 0x89, 0x8a,
0xab, 0xa8, 0xad, 0xae, 0xa7, 0xa4, 0xa1, 0xa2, 0xb3, 0xb0, 0xb5, 0xb6, 0xbf, 0xbc, 0xb9, 0xba,
0xfb, 0xf8, 0xfd, 0xfe, 0xf7, 0xf4, 0xf1, 0xf2, 0xe3, 0xe0, 0xe5, 0xe6, 0xef, 0xec, 0xe9, 0xea,
0xcb, 0xc8, 0xcd, 0xce, 0xc7, 0xc4, 0xc1, 0xc2, 0xd3, 0xd0, 0xd5, 0xd6, 0xdf, 0xdc, 0xd9, 0xda,
0x5b, 0x58, 0x5d, 0x5e, 0x57, 0x54, 0x51, 0x52, 0x43, 0x40, 0x45, 0x46, 0x4f, 0x4c, 0x49, 0x4a,
0x6b, 0x68, 0x6d, 0x6e, 0x67, 0x64, 0x61, 0x62, 0x73, 0x70, 0x75, 0x76, 0x7f, 0x7c, 0x79, 0x7a,
0x3b, 0x38, 0x3d, 0x3e, 0x37, 0x34, 0x31, 0x32, 0x23, 0x20, 0x25, 0x26, 0x2f, 0x2c, 0x29, 0x2a,
0x0b, 0x08, 0x0d, 0x0e, 0x07, 0x04, 0x01, 0x02, 0x13, 0x10, 0x15, 0x16, 0x1f, 0x1c, 0x19, 0x1a};

// 
void MixColumns(uint8 state[][Nb])
{
    uint8 temp[4];
    int i, j;

    for (i = 0; i <= Nb - 1; i++)
    {
        for (j = 0; j <= 3; j++)
            temp[j] = state[j][i];
        state[0][i] = (uint8)(X_mul_2[temp[0]] ^ X_mul_3[temp[1]] ^         temp[2]  ^         temp[3]);
        state[1][i] = (uint8)(        temp[0]  ^ X_mul_2[temp[1]] ^ X_mul_3[temp[2]] ^         temp[3]);
        state[2][i] = (uint8)(        temp[0]  ^         temp[1]  ^ X_mul_2[temp[2]] ^ X_mul_3[temp[3]]);
        state[3][i] = (uint8)(X_mul_3[temp[0]] ^         temp[1]  ^         temp[2]  ^ X_mul_2[temp[3]]);
    }
}

//     (i-  
// i )
void InvShiftRows(uint8 state[][Nb])
{
    uint8 temp[Nb];
    int i, j, n;

    for (i = 1; i <= 3; i++)
    {
        for (j = 0; j <= Nb - 1; j++)
            temp[j] = state[i][j];
        for (j = 0; j <= Nb - 1; j++)
        {
            n = (j + i > Nb - 1) ? (j + i - Nb) : (j + i);
            state[i][n] = temp[j];
        }
    }    
}

//,      
static const uint8 inv_S_box[256] = {
0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3, 0x9e, 0x81, 0xf3, 0xd7, 0xfb,
0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f, 0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb,
0x54, 0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b, 0x42, 0xfa, 0xc3, 0x4e,
0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24, 0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25,
0x72, 0xf8, 0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d, 0x65, 0xb6, 0x92,
0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda, 0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84,
0x90, 0xd8, 0xab, 0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3, 0x45, 0x06,
0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1, 0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b,
0x3a, 0x91, 0x11, 0x41, 0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6, 0x73,
0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9, 0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e,
0x47, 0xf1, 0x1a, 0x71, 0x1d, 0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b,
0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0, 0xfe, 0x78, 0xcd, 0x5a, 0xf4,
0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07, 0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f,
0x60, 0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f, 0x93, 0xc9, 0x9c, 0xef,
0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5, 0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61,
0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d};

//       inv_S_box
void InvSubBytes(uint8 state[][Nb])
{
    int i, j;

    for (i = 0; i <= Nb - 1; i++)
        for (j = 0; j <= 3; j++)
            state[j][i] = inv_S_box[state[j][i]];
}

//   x  0x0E   GF(2^8)
static const uint8 X_mul_E[256] = {
0x00, 0x0e, 0x1c, 0x12, 0x38, 0x36, 0x24, 0x2a, 0x70, 0x7e, 0x6c, 0x62, 0x48, 0x46, 0x54, 0x5a,
0xe0, 0xee, 0xfc, 0xf2, 0xd8, 0xd6, 0xc4, 0xca, 0x90, 0x9e, 0x8c, 0x82, 0xa8, 0xa6, 0xb4, 0xba,
0xdb, 0xd5, 0xc7, 0xc9, 0xe3, 0xed, 0xff, 0xf1, 0xab, 0xa5, 0xb7, 0xb9, 0x93, 0x9d, 0x8f, 0x81,
0x3b, 0x35, 0x27, 0x29, 0x03, 0x0d, 0x1f, 0x11, 0x4b, 0x45, 0x57, 0x59, 0x73, 0x7d, 0x6f, 0x61,
0xad, 0xa3, 0xb1, 0xbf, 0x95, 0x9b, 0x89, 0x87, 0xdd, 0xd3, 0xc1, 0xcf, 0xe5, 0xeb, 0xf9, 0xf7,
0x4d, 0x43, 0x51, 0x5f, 0x75, 0x7b, 0x69, 0x67, 0x3d, 0x33, 0x21, 0x2f, 0x05, 0x0b, 0x19, 0x17,
0x76, 0x78, 0x6a, 0x64, 0x4e, 0x40, 0x52, 0x5c, 0x06, 0x08, 0x1a, 0x14, 0x3e, 0x30, 0x22, 0x2c,
0x96, 0x98, 0x8a, 0x84, 0xae, 0xa0, 0xb2, 0xbc, 0xe6, 0xe8, 0xfa, 0xf4, 0xde, 0xd0, 0xc2, 0xcc,
0x41, 0x4f, 0x5d, 0x53, 0x79, 0x77, 0x65, 0x6b, 0x31, 0x3f, 0x2d, 0x23, 0x09, 0x07, 0x15, 0x1b,
0xa1, 0xaf, 0xbd, 0xb3, 0x99, 0x97, 0x85, 0x8b, 0xd1, 0xdf, 0xcd, 0xc3, 0xe9, 0xe7, 0xf5, 0xfb,
0x9a, 0x94, 0x86, 0x88, 0xa2, 0xac, 0xbe, 0xb0, 0xea, 0xe4, 0xf6, 0xf8, 0xd2, 0xdc, 0xce, 0xc0,
0x7a, 0x74, 0x66, 0x68, 0x42, 0x4c, 0x5e, 0x50, 0x0a, 0x04, 0x16, 0x18, 0x32, 0x3c, 0x2e, 0x20,
0xec, 0xe2, 0xf0, 0xfe, 0xd4, 0xda, 0xc8, 0xc6, 0x9c, 0x92, 0x80, 0x8e, 0xa4, 0xaa, 0xb8, 0xb6,
0x0c, 0x02, 0x10, 0x1e, 0x34, 0x3a, 0x28, 0x26, 0x7c, 0x72, 0x60, 0x6e, 0x44, 0x4a, 0x58, 0x56,
0x37, 0x39, 0x2b, 0x25, 0x0f, 0x01, 0x13, 0x1d, 0x47, 0x49, 0x5b, 0x55, 0x7f, 0x71, 0x63, 0x6d,
0xd7, 0xd9, 0xcb, 0xc5, 0xef, 0xe1, 0xf3, 0xfd, 0xa7, 0xa9, 0xbb, 0xb5, 0x9f, 0x91, 0x83, 0x8d};

//   x  0x0B   GF(2^8)
static const uint8 X_mul_B[256] = {
0x00, 0x0b, 0x16, 0x1d, 0x2c, 0x27, 0x3a, 0x31, 0x58, 0x53, 0x4e, 0x45, 0x74, 0x7f, 0x62, 0x69,
0xb0, 0xbb, 0xa6, 0xad, 0x9c, 0x97, 0x8a, 0x81, 0xe8, 0xe3, 0xfe, 0xf5, 0xc4, 0xcf, 0xd2, 0xd9,
0x7b, 0x70, 0x6d, 0x66, 0x57, 0x5c, 0x41, 0x4a, 0x23, 0x28, 0x35, 0x3e, 0x0f, 0x04, 0x19, 0x12,
0xcb, 0xc0, 0xdd, 0xd6, 0xe7, 0xec, 0xf1, 0xfa, 0x93, 0x98, 0x85, 0x8e, 0xbf, 0xb4, 0xa9, 0xa2,
0xf6, 0xfd, 0xe0, 0xeb, 0xda, 0xd1, 0xcc, 0xc7, 0xae, 0xa5, 0xb8, 0xb3, 0x82, 0x89, 0x94, 0x9f,
0x46, 0x4d, 0x50, 0x5b, 0x6a, 0x61, 0x7c, 0x77, 0x1e, 0x15, 0x08, 0x03, 0x32, 0x39, 0x24, 0x2f,
0x8d, 0x86, 0x9b, 0x90, 0xa1, 0xaa, 0xb7, 0xbc, 0xd5, 0xde, 0xc3, 0xc8, 0xf9, 0xf2, 0xef, 0xe4,
0x3d, 0x36, 0x2b, 0x20, 0x11, 0x1a, 0x07, 0x0c, 0x65, 0x6e, 0x73, 0x78, 0x49, 0x42, 0x5f, 0x54,
0xf7, 0xfc, 0xe1, 0xea, 0xdb, 0xd0, 0xcd, 0xc6, 0xaf, 0xa4, 0xb9, 0xb2, 0x83, 0x88, 0x95, 0x9e,
0x47, 0x4c, 0x51, 0x5a, 0x6b, 0x60, 0x7d, 0x76, 0x1f, 0x14, 0x09, 0x02, 0x33, 0x38, 0x25, 0x2e,
0x8c, 0x87, 0x9a, 0x91, 0xa0, 0xab, 0xb6, 0xbd, 0xd4, 0xdf, 0xc2, 0xc9, 0xf8, 0xf3, 0xee, 0xe5,
0x3c, 0x37, 0x2a, 0x21, 0x10, 0x1b, 0x06, 0x0d, 0x64, 0x6f, 0x72, 0x79, 0x48, 0x43, 0x5e, 0x55,
0x01, 0x0a, 0x17, 0x1c, 0x2d, 0x26, 0x3b, 0x30, 0x59, 0x52, 0x4f, 0x44, 0x75, 0x7e, 0x63, 0x68,
0xb1, 0xba, 0xa7, 0xac, 0x9d, 0x96, 0x8b, 0x80, 0xe9, 0xe2, 0xff, 0xf4, 0xc5, 0xce, 0xd3, 0xd8,
0x7a, 0x71, 0x6c, 0x67, 0x56, 0x5d, 0x40, 0x4b, 0x22, 0x29, 0x34, 0x3f, 0x0e, 0x05, 0x18, 0x13,
0xca, 0xc1, 0xdc, 0xd7, 0xe6, 0xed, 0xf0, 0xfb, 0x92, 0x99, 0x84, 0x8f, 0xbe, 0xb5, 0xa8, 0xa3};

//   x  0x0D   GF(2^8)
static const uint8 X_mul_D[256] = {
0x00, 0x0d, 0x1a, 0x17, 0x34, 0x39, 0x2e, 0x23, 0x68, 0x65, 0x72, 0x7f, 0x5c, 0x51, 0x46, 0x4b,
0xd0, 0xdd, 0xca, 0xc7, 0xe4, 0xe9, 0xfe, 0xf3, 0xb8, 0xb5, 0xa2, 0xaf, 0x8c, 0x81, 0x96, 0x9b,
0xbb, 0xb6, 0xa1, 0xac, 0x8f, 0x82, 0x95, 0x98, 0xd3, 0xde, 0xc9, 0xc4, 0xe7, 0xea, 0xfd, 0xf0,
0x6b, 0x66, 0x71, 0x7c, 0x5f, 0x52, 0x45, 0x48, 0x03, 0x0e, 0x19, 0x14, 0x37, 0x3a, 0x2d, 0x20,
0x6d, 0x60, 0x77, 0x7a, 0x59, 0x54, 0x43, 0x4e, 0x05, 0x08, 0x1f, 0x12, 0x31, 0x3c, 0x2b, 0x26,
0xbd, 0xb0, 0xa7, 0xaa, 0x89, 0x84, 0x93, 0x9e, 0xd5, 0xd8, 0xcf, 0xc2, 0xe1, 0xec, 0xfb, 0xf6,
0xd6, 0xdb, 0xcc, 0xc1, 0xe2, 0xef, 0xf8, 0xf5, 0xbe, 0xb3, 0xa4, 0xa9, 0x8a, 0x87, 0x90, 0x9d,
0x06, 0x0b, 0x1c, 0x11, 0x32, 0x3f, 0x28, 0x25, 0x6e, 0x63, 0x74, 0x79, 0x5a, 0x57, 0x40, 0x4d,
0xda, 0xd7, 0xc0, 0xcd, 0xee, 0xe3, 0xf4, 0xf9, 0xb2, 0xbf, 0xa8, 0xa5, 0x86, 0x8b, 0x9c, 0x91,
0x0a, 0x07, 0x10, 0x1d, 0x3e, 0x33, 0x24, 0x29, 0x62, 0x6f, 0x78, 0x75, 0x56, 0x5b, 0x4c, 0x41,
0x61, 0x6c, 0x7b, 0x76, 0x55, 0x58, 0x4f, 0x42, 0x09, 0x04, 0x13, 0x1e, 0x3d, 0x30, 0x27, 0x2a,
0xb1, 0xbc, 0xab, 0xa6, 0x85, 0x88, 0x9f, 0x92, 0xd9, 0xd4, 0xc3, 0xce, 0xed, 0xe0, 0xf7, 0xfa,
0xb7, 0xba, 0xad, 0xa0, 0x83, 0x8e, 0x99, 0x94, 0xdf, 0xd2, 0xc5, 0xc8, 0xeb, 0xe6, 0xf1, 0xfc,
0x67, 0x6a, 0x7d, 0x70, 0x53, 0x5e, 0x49, 0x44, 0x0f, 0x02, 0x15, 0x18, 0x3b, 0x36, 0x21, 0x2c,
0x0c, 0x01, 0x16, 0x1b, 0x38, 0x35, 0x22, 0x2f, 0x64, 0x69, 0x7e, 0x73, 0x50, 0x5d, 0x4a, 0x47,
0xdc, 0xd1, 0xc6, 0xcb, 0xe8, 0xe5, 0xf2, 0xff, 0xb4, 0xb9, 0xae, 0xa3, 0x80, 0x8d, 0x9a, 0x97};

//   x  0x09   GF(2^8)
static const uint8 X_mul_9[256] = {
0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f, 0x48, 0x41, 0x5a, 0x53, 0x6c, 0x65, 0x7e, 0x77,
0x90, 0x99, 0x82, 0x8b, 0xb4, 0xbd, 0xa6, 0xaf, 0xd8, 0xd1, 0xca, 0xc3, 0xfc, 0xf5, 0xee, 0xe7,
0x3b, 0x32, 0x29, 0x20, 0x1f, 0x16, 0x0d, 0x04, 0x73, 0x7a, 0x61, 0x68, 0x57, 0x5e, 0x45, 0x4c,
0xab, 0xa2, 0xb9, 0xb0, 0x8f, 0x86, 0x9d, 0x94, 0xe3, 0xea, 0xf1, 0xf8, 0xc7, 0xce, 0xd5, 0xdc,
0x76, 0x7f, 0x64, 0x6d, 0x52, 0x5b, 0x40, 0x49, 0x3e, 0x37, 0x2c, 0x25, 0x1a, 0x13, 0x08, 0x01,
0xe6, 0xef, 0xf4, 0xfd, 0xc2, 0xcb, 0xd0, 0xd9, 0xae, 0xa7, 0xbc, 0xb5, 0x8a, 0x83, 0x98, 0x91,
0x4d, 0x44, 0x5f, 0x56, 0x69, 0x60, 0x7b, 0x72, 0x05, 0x0c, 0x17, 0x1e, 0x21, 0x28, 0x33, 0x3a,
0xdd, 0xd4, 0xcf, 0xc6, 0xf9, 0xf0, 0xeb, 0xe2, 0x95, 0x9c, 0x87, 0x8e, 0xb1, 0xb8, 0xa3, 0xaa,
0xec, 0xe5, 0xfe, 0xf7, 0xc8, 0xc1, 0xda, 0xd3, 0xa4, 0xad, 0xb6, 0xbf, 0x80, 0x89, 0x92, 0x9b,
0x7c, 0x75, 0x6e, 0x67, 0x58, 0x51, 0x4a, 0x43, 0x34, 0x3d, 0x26, 0x2f, 0x10, 0x19, 0x02, 0x0b,
0xd7, 0xde, 0xc5, 0xcc, 0xf3, 0xfa, 0xe1, 0xe8, 0x9f, 0x96, 0x8d, 0x84, 0xbb, 0xb2, 0xa9, 0xa0,
0x47, 0x4e, 0x55, 0x5c, 0x63, 0x6a, 0x71, 0x78, 0x0f, 0x06, 0x1d, 0x14, 0x2b, 0x22, 0x39, 0x30,
0x9a, 0x93, 0x88, 0x81, 0xbe, 0xb7, 0xac, 0xa5, 0xd2, 0xdb, 0xc0, 0xc9, 0xf6, 0xff, 0xe4, 0xed,
0x0a, 0x03, 0x18, 0x11, 0x2e, 0x27, 0x3c, 0x35, 0x42, 0x4b, 0x50, 0x59, 0x66, 0x6f, 0x74, 0x7d,
0xa1, 0xa8, 0xb3, 0xba, 0x85, 0x8c, 0x97, 0x9e, 0xe9, 0xe0, 0xfb, 0xf2, 0xcd, 0xc4, 0xdf, 0xd6,
0x31, 0x38, 0x23, 0x2a, 0x15, 0x1c, 0x07, 0x0e, 0x79, 0x70, 0x6b, 0x62, 0x5d, 0x54, 0x4f, 0x46};

// 
void InvMixColumns(uint8 state[][Nb])
{
    uint8 temp[4];
    int i, j;

    for (i = 0; i <= Nb - 1; i++)
    {
        for (j = 0; j <= 3; j++)
            temp[j] = state[j][i];
        state[0][i] = (uint8)(X_mul_E[temp[0]] ^ X_mul_B[temp[1]] ^ X_mul_D[temp[2]] ^ X_mul_9[temp[3]]);
        state[1][i] = (uint8)(X_mul_9[temp[0]] ^ X_mul_E[temp[1]] ^ X_mul_B[temp[2]] ^ X_mul_D[temp[3]]);
        state[2][i] = (uint8)(X_mul_D[temp[0]] ^ X_mul_9[temp[1]] ^ X_mul_E[temp[2]] ^ X_mul_B[temp[3]]);
        state[3][i] = (uint8)(X_mul_B[temp[0]] ^ X_mul_D[temp[1]] ^ X_mul_9[temp[2]] ^ X_mul_E[temp[3]]);
    }
}



//////////////////////// MAC ////////////////////////////////



void xor_128(BYTE *a, BYTE *b, BYTE *out)
{
    int i;
    for (i=0;i<16; i++)
        out[i] = a[i] ^ b[i];
}

void xor_192(BYTE *a, BYTE *b, BYTE *out)
{
    int i;
    for (i=0;i<24; i++)
        out[i] = a[i] ^ b[i];
}

void xor_size(BYTE *a, BYTE *b, BYTE *out, BYTE Size = 16)
{
    int i;
    for (i=0;i<Size; i++)
        out[i] = a[i] ^ b[i];
}

void leftshift_onebit(BYTE *input, BYTE *output, BYTE Size = 16)
{
    int i;
    BYTE overflow = 0;

    for ( i=Size-1; i>=0; i-- ) 
    {
        output[i] = input[i] << 1;
        output[i] |= overflow;
        overflow = (input[i] & 0x80)?1:0;
    }
    
    return;
}

void generate_subkey(BYTE *key, BYTE *K1, BYTE *K2)
{
	BYTE const_Rb[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0x87};

	BYTE L[16];
    BYTE Z[16];
    BYTE tmp[16];
    int i;

    for ( i=0; i<16; i++ ) 
        Z[i] = 0;

    aes_128_encrypt16( Z, key, L );

    if ( (L[0] & 0x80) == 0 ) { /* If MSB(L) = 0, then K1 = L << 1 */
        leftshift_onebit(L,K1);
    } else { /* Else K1 = ( L << 1 ) (+) Rb */

        leftshift_onebit(L,tmp);
        xor_128(tmp,const_Rb,K1);
    }

    if ( (K1[0] & 0x80) == 0 ) 
    {
        leftshift_onebit(K1,K2);
    } else 
    {
        leftshift_onebit(K1,tmp);
        xor_128(tmp,const_Rb,K2);
    }
    
    return;
}

void AES_128_CBC_En_MAC( unsigned char* pInp, int ByteSize, unsigned char* pKey, unsigned char* pOut )
{
// Parameters: BYTE* pInp, int ByteSize, BYTE* pKey, BYTE* pOut
    int i, j, BlockQty;
    BYTE* pPlain;
    BYTE* pSubKey;
    BYTE wrk[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    BYTE pK1[16];
    BYTE pK2[16];

    if( ByteSize & 0x0f )
    {
        pSubKey = pK2;
        pInp[ByteSize] = 0x80;
        for( i = ByteSize + 1; ( i & 0x0f ) > 0; i++ )
            pInp[i] = 0;
    }
    else
        pSubKey = pK1;
    BlockQty = ( ByteSize + 15 ) >> 4;
    generate_subkey( pKey, pK1, pK2 );
    pPlain = pInp;
    for( i = 0; i < BlockQty; i++ )
    {
        for( j = 0; j < 16; j++ )
        {
            wrk[j] ^= pPlain[j]; 
            if( i == BlockQty - 1 )
//                wrk[j] ^= pK2[j]; 
                wrk[j] ^= pSubKey[j]; 
        }
        aes_128_encrypt16( wrk, pKey, pOut );
        memcpy( wrk, pOut, 16 );
        pPlain += 16;
    }
}

/*
void AES_128_CBC_En_MAC( BYTE* pInp, int BlockQty, BYTE* pKey, BYTE* pOut, BYTE* pIV )
{
    int i, j;
    BYTE *pPlain;
    BYTE wrk[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    BYTE pK1[16];
    BYTE pK2[16];
    pPlain = pInp;

    generate_subkey(pKey, pK1, pK2);
    
	if (pIV)
		xor_128(wrk, pIV, wrk);

    for( i = 0; i < BlockQty - 1; i++ )
    {
		xor_128(wrk, pPlain, wrk);
        aes_128_encrypt16( wrk, pKey, pOut );
        memcpy( wrk, pOut, 16 );
        pPlain += 16;
    }

	xor_128(wrk, pPlain, wrk);
	xor_128(wrk, pK2, wrk);
    aes_128_encrypt16( wrk, pKey, pOut );
}
*/
void AES_192_CBC_En_MAC( BYTE* pInp, int BlockQty, BYTE* pKey, BYTE* pOut, BYTE* pIV )
{
    int i;
    BYTE *pPlain;
    BYTE wrk[24] = {0};
    BYTE pK1[24];
    BYTE pK2[24];
    pPlain = pInp;

    generate_subkey(pKey, pK1, pK2);
    
	if (pIV)
		xor_192(wrk, pIV, wrk);

    for( i = 0; i < BlockQty - 1; i++ )
    {
		xor_192(wrk, pPlain, wrk);
        aes_192_encrypt24( wrk, pKey, pOut );
        memcpy( wrk, pOut, 24 );
        pPlain += 24;
    }

	xor_192(wrk, pPlain, wrk);
	xor_192(wrk, pK2, wrk);
    aes_192_encrypt24( wrk, pKey, pOut );
}